/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.Animation = Class.$define("APT.Animation", {
    __init__: function() {
        this._id = null;
        this._duration = 0;
        this._delay = 0;
        this.parameters = new Array();
        this.views = new Array();
        this._context = null;
        this._animationsFinishedCount = 0;
        this._willStartSelector = null;
        this._didStopSelector = null;
        this._delegate = null;
        this._viewTransition = null;
        this.viewAdds = new Array();
        this.viewAddsReference = new Array();
        this.viewRemoves = new Array();
        this._rotYINT = 0;
        this._transition = 0;
    },

    $copy: function() {
        var copyObject = new APT.Animation();
        copyObject._id = this._id;
        copyObject._duration = this._duration;
        copyObject._delay = this._delay;
        copyObject.parameters = this.parameters;
        copyObject.views = this.views;
        copyObject._context = this._context;
        copyObject._animationsFinishedCount = this._animationsFinishedCount;
        copyObject._willStartSelector = this._willStartSelector;
        copyObject._didStopSelector = this._didStopSelector;
        copyObject._delegate = this._delegate;
        copyObject._viewTransition = this._viewTransition;
        copyObject.viewAdds = this.viewAdds;
        copyObject.viewAddsReference = this.viewAddsReference;
        copyObject.viewRemoves = this.viewRemoves;
        copyObject._rotYINT = this._rotYINT;
        copyObject._transition = this._transition;
        return copyObject;
    },

    setAnimatableProperty: function(aView, aPropertyName, aValue) {
        var index = this.views.indexOf(aView);
        if (index === -1)
        {
            index = this.views.length;
            this.parameters.push(APT.Animation.Parameters());
            this.views.push(aView);
        }

        this.parameters[index][aPropertyName] = aValue;
    },

    animateAlpha: function(aView, aValue) {
        this.setAnimatableProperty(aView, "opacity", aValue);
    },

    animateFrame: function(aView, aValue) {
        aValue = aValue.$copy();
        this.setAnimatableProperty(aView, "left", aValue.origin.x);
        this.setAnimatableProperty(aView, "top", aValue.origin.y);
        this.setAnimatableProperty(aView, "width", aValue.size.width);
        this.setAnimatableProperty(aView, "height", aValue.size.height);
    },

    animateBackground: function(aView, aValue) {
        this.setAnimatableProperty(aView, "backgroundColor", aValue.getHtmlValue());
    },

    commitAnimations: function() {
        this.animationsWillStart();

        if (this._delay > 0)
        {
            var animation = this;

            window.setTimeout(function() {
                animation.initAnimation();
            }, this._delay);
        }
        else
        {
            this.initAnimation();
        }

        if (this.views.length === 0 && this._transition !== 0)
        {
            this.startFlip();
        }
    },

    setAnimationDelegate: function(aDelegate) {
        this._delegate = aDelegate;
    },

    animationDelegate: function() {
        return this._delegate;
    },

    animationsWillStart: function() {
        if (this._delegate !== null && this._willStartSelector !== null)
        {
            (this._delegate[this._willStartSelector]).call(this._delegate, this._id, this._context);
        }
    },

    setAnimationDelay: function(aDelay) {
        this._delay = aDelay;
    },

    setAnimationDuration: function(aDuration) {
        this._duration = aDuration;
    },

    setAnimationWillStartSelector: function(selector) {
        this._willStartSelector = selector;
    },

    setAnimationDidStopSelector: function(selector) {
        this._didStopSelector = selector;
    },

    setAnimationTransitionForViewCache: function(aTransition, aView) {
        this._transition = aTransition;
        this._viewTransition = aView;
    },

    animateAddSubview: function(aView, aViewAdd) {
        this.viewAddsReference.push(aView);
        this.viewAdds.push(aViewAdd);
    },

    animateRemove: function(aViewRemove) {
        this.viewRemoves.push(aViewRemove);
    },

    initAnimation: function() {
        this._animationsFinishedCount = this.views.length;
        var animation = this;
        var eventHandler = function() {
            animation._animationsFinishedCount--;
            if (animation._animationsFinishedCount === 0)
            {
                if (animation._delegate !== null && animation._didStopSelector !== null)
                {
                    (animation._delegate[animation._didStopSelector]).call(animation._delegate, animation._id, true, animation._context);
                }
            }
        };

        for (var i = 0; i < this.views.length; i++)
        {
            var view = this.views[i];

            if (this._transition !== 0)
            {
                this.startFlip();
            }

            var options = new Array();
            options["queue"] = false;
            options["duration"] = this._duration;
            options["complete"] = eventHandler;

            view.jQElement().animate(this.parameters[i], options);
        }
    },

    startFlip: function() {
        window.clearInterval(this._rotYINT);
        var animation = this;

        var change = true;
        var ny = 0;

        this._rotYINT = window.setInterval(function() {
            if (animation._transition === APT.AnimationTransition.FlipFromLeft)
            {
                if (ny < 90 && change)
                {
                    ny = ny + 1;
                }
                else
                {
                    change = false;
                    ny = ny - 1;
                }

                animation.flipFromLeft(ny);
            }
            else
            {
                if (animation._transition === APT.AnimationTransition.FlipFromRight)
                {
                    if (ny > -90 && change)
                    {
                        ny = ny - 1;
                    }
                    else
                    {
                        change = false;
                        ny = ny + 1;
                    }

                    animation.flipFromRight(ny);
                }
            }
        }, this._duration / 200);
    },

    flipFromLeft: function(degree) {
        var rotate = "rotateY(" + degree + "deg)";

        this._viewTransition.jQElement().css("transform", rotate);
        this._viewTransition.jQElement().css("webkitTransform", rotate);
        this._viewTransition.jQElement().css("OTransform", rotate);
        this._viewTransition.jQElement().css("MozTransform", rotate);

        if (degree === 90)
        {
            for (var i = 0; i < this.viewRemoves.length; i++)
            {
                (this.viewRemoves[i]).removeFromSuperview();
            }

            for (var i = 0; i < this.viewAdds.length; i++)
            {
                (this.viewAddsReference[i]).addSubview(this.viewAdds[i]);
            }
        }

        if (degree === 0)
        {
            window.clearInterval(this._rotYINT);
        }
    },

    flipFromRight: function(degree) {
        var rotate = "rotateY(" + degree + "deg)";

        this._viewTransition.jQElement().css("transform", rotate);
        this._viewTransition.jQElement().css("webkitTransform", rotate);
        this._viewTransition.jQElement().css("OTransform", rotate);
        this._viewTransition.jQElement().css("MozTransform", rotate);

        if (degree === -90)
        {
            for (var i = 0; i < this.viewRemoves.length; i++)
            {
                (this.viewRemoves[i]).removeFromSuperview();
            }

            for (var i = 0; i < this.viewAdds.length; i++)
            {
                (this.viewAddsReference[i]).addSubview(this.viewAdds[i]);
            }
        }

        if (degree === 0)
        {
            window.clearInterval(this._rotYINT);
        }
    },

});

APT.Animation.Parameters = function() {
    return new Array();
};

APT.Animation.beginAnimationsContext = function(key, userInfo) {
    var animation = new APT.Animation();
    animation._id = key;
    animation._context = userInfo;
    return animation;
};


