/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");
APT.Control = Class.$define("APT.Control", APT.View);

APT.Button = Class.$define("APT.Button", APT.Control, {
    __init__: function() {
        this._label = null;
        this._imageView = null;
        this._imageSizeCounter = 0;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.Button();
        copyObject._label = this._label;
        copyObject._imageView = this._imageView;
        copyObject._imageSizeCounter = this._imageSizeCounter;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<div><button data-button-type='0'></button></div>"));
    },

    setJQElement: function(aObject) {
        var text = aObject.text();
        var buttonType = parseInt(aObject.find("button").attr("data-button-type"));
        this._jqElement = aObject.trigger("create");
        var imageName = aObject.find("button").attr("data-apt-image");

        this._imageView = new APT.ImageView();

        this._label = new APT.Label();
        this._label.setBackgroundColor(APT.Color.Transparent);
        this._label.setTextAlignment(APT.TextAlignment.Center);
        if (text.length > 0)
        {
            this._label.setText(text);
            this.jQElement().find("button").text("");
        }

        switch(buttonType)
        {

            case APT.ButtonType.InfoLight:
            case APT.ButtonType.InfoDark:
                this.jQElement().find("button").attr("data-icon", "info");
                if (aObject.find("button").attr("data-iconpos") === undefined || aObject.find("button").attr("data-iconpos").length === 0)
                {
                    this.jQElement().find("button").attr("data-iconpos", "left");
                }

                this.jQElement().find("button").attr("data-corners", "false");
                this.jQElement().find("button").attr("data-shadow", "false");
                break;
            case APT.ButtonType.RoundedRect:
                break;
            case APT.ButtonType.Add:
                this.jQElement().find("button").attr("data-icon", "plus");
                if (aObject.find("button").attr("data-iconpos") === undefined || aObject.find("button").attr("data-iconpos").length === 0)
                {
                    this.jQElement().find("button").attr("data-iconpos", "left");
                }

            case APT.ButtonType.Custom:
            case APT.ButtonType.DetailDisclosure:
            default:
                this.jQElement().find("button").attr("data-corners", "false");
                this.jQElement().find("button").attr("data-shadow", "false");
                break;
        }

        this.jQElement().find("button").attr("data-theme", "c");
        this.jQElement().find("button").button();
        this.jQElement().find(".ui-btn-inner").css("padding", 0);
        this.jQElement().find(".ui-btn-inner").css("border-top", 0);
        this.jQElement().find(".ui-btn-inner").append(this._label.jQElement());
        if (imageName !== undefined && imageName.length > 0)
        {
            this.setImage(APT.Image.imageNamed(imageName));
        }

        if (aObject.find("button").attr("data-iconpos") === "notext")
        {
            this.jQElement().find(".ui-btn-inner").css("display", "inline-table");
        }

        this.standardizeMarginAndPaddingFormInnerSpan();
        this.jQElement().addClass("apt_button_background");
        this.jQElement().find(".ui-btn").addClass("apt_button");
        this.jQElement().find(".ui-btn").addClass("apt_font");
        this.setProperties();
    },

    buttonType: function() {
        return parseInt(this.jQElement().find("button").attr("data-button-type"));
    },

    setTitle: function(titleText) {
        if (this._label !== null)
        {
            this._label.setText(titleText);
            this.setNeedsLayout();
        }
    },

    title: function() {
        var result = null;
        if (this._label !== null)
        {
            result = this._label.text();
        }

        return result;
    },

    setImage: function(aImage) {
        if (this._imageView !== null)
        {
            if (this._imageView.image() === null)
            {
                this._imageView.setImage(aImage);
                this.jQElement().find(".ui-btn-inner").append(this._imageView.jQElement());
            }
            else
            {
                this._imageView.setImage(aImage);
            }

            this._imageSizeCounter = 0;
            this.setNeedsLayout();
        }
    },

    image: function() {
        var result = null;
        if (this._imageView !== null)
        {
            result = this._imageView.image();
        }

        return result;
    },

    imageView: function() {
        return this._imageView;
    },

    titleLabel: function() {
        return this._label;
    },

    setTitleColor: function(titleColor) {
        if (this._label !== null)
        {
            this._label.setTextColor(titleColor);
        }
    },

    titleColor: function() {
        var result = null;
        if (this._label !== null)
        {
            result = this._label.textColor();
        }

        return result;
    },

    font: function() {
        var result = null;
        if (this._label !== null)
        {
            result = this._label.font();
        }

        return result;
    },

    setFont: function(aFont) {
        if (this._label !== null)
        {
            this._label.setFont(aFont);
        }
    },

    text: function() {
        var result = null;
        if (this._label !== null)
        {
            result = this._label.text();
        }

        return result;
    },

    setText: function(aTitle) {
        if (this._label !== null)
        {
            this._label.setText(aTitle);
        }
    },

    setBackgroundColor: function(aColor) {
        this.$super(APT.Color.Transparent);
        if (aColor !== null)
        {
            this.jQElement().find(".ui-btn").css("background-color", aColor.getHtmlValue().valueOf());
            if (this.buttonType() === APT.ButtonType.Custom)
            {
                this.jQElement().find(".ui-btn").css("background", aColor.getHtmlValue().valueOf());
            }
        }

        this.jQElement().find(".ui-btn").css("border", "0px solid #CCC");
    },

    backgroundColor: function() {
        if (this._backgroundColor !== null)
        {
            this._backgroundColor.updateFromCSS(this.jQElement().find(".ui-btn").css("background-color"));
            return this._backgroundColor;
        }

        return null;
    },

    viewDidAppear: function(animated) {
        this.setNeedsLayout();

        this._label.viewDidAppear(animated);
        this._imageView.viewDidAppear(animated);
        this.$super(animated);
    },

    layoutSubviews: function() {
        this.$super();

        this.standardizeMarginAndPaddingFormInnerSpan();
        if (this.frame().size.width > 0)
        {
            APT.Size.updateCSS(this.jQElement().find(".ui-btn-inner"), this.frame().size);
            APT.Size.updateCSS(this.jQElement().find(".ui-btn-text"), this.frame().size);
        }

        if (this._imageView !== null)
        {
            this.adjustImageSize();
        }

        if (this._label !== null)
        {
            this.adjustLabelSize();
        }
    },

    setFrame: function(aFrame) {
        aFrame = aFrame.$copy();
        if (this.frame().size.width > 0)
        {
            APT.Size.updateCSS(this.jQElement().find(".ui-btn-inner"), aFrame.size);
            APT.Size.updateCSS(this.jQElement().find(".ui-btn-text"), aFrame.size);
        }

        this.$super(aFrame);
    },

    insertSubviewAtIndexWithoutNotifying: function(aView, anIndex) {
        var len = this._subviews.length;
        if (aView !== null && aView !== this && anIndex >= 0 && anIndex <= len)
        {
            if (aView.superview() !== null)
            {
                aView.removeFromSuperviewWithoutNotifying();
            }

            if (anIndex === len)
            {
                this._jqElement.find(".ui-btn-text").append(aView.jQElement());
                this._subviews.push(aView);
            }
            else
            {
                aView.jQElement().find(".ui-btn-text").insertBefore((this._subviews[anIndex]).jQElement());
                this._subviews.splice(anIndex, 0, aView);
            }

            aView._superview = this;
            aView.setNeedsLayout();
        }
    },

    processNode: function(markup, element) {
        var buttonType = parseInt(element.find("button").attr("data-button-type"));
        var text = element.find("button").text();

        if (buttonType === APT.ButtonType.Add || buttonType === APT.ButtonType.InfoLight || buttonType === APT.ButtonType.InfoDark && text.length === 0)
        {
            markup.find("button").attr("data-iconpos", "notext");
        }

        markup.find("button").attr("data-button-type", String(buttonType));
        markup.find("button").attr("data-apt-image", element.find("button").attr("data-apt-image"));
        markup.find("button").text(text);

        this.setJQElement(markup);
    },

    adjustImageSize: function() {
        if (this.imageView() !== null && this.frame().size.height > 0)
        {
            var buttonHeight = this.frame().size.height;
            var buttonWidth = this.frame().size.width;
            var imgHeight = 0;
            var marginTopImage = 0;
            var imgWidth = 0;
            var imgX = 10;
            if (this.imageView().image() !== null)
            {
                imgHeight = this.imageView().image().size().height;
                imgWidth = this.imageView().image().size().width;

                if (imgWidth >= buttonWidth)
                {
                    imgWidth = buttonWidth;
                    imgX = 0;
                }

                if (buttonHeight < imgHeight)
                {
                    imgHeight = buttonHeight;
                }
                else
                {
                    if (imgHeight > 0)
                    {
                        marginTopImage = (buttonHeight - imgHeight) / 2;
                    }
                    else
                    {
                        if (this.image() !== null && this._imageSizeCounter++ < 10)
                        {
                            var thisButton = this;
                            window.setTimeout(function() {
                                thisButton.setNeedsLayout();
                            }, 100);
                        }
                    }
                }

                if (imgHeight > 0 && imgWidth > 0)
                {
                    this._imageView.setFrame(APT.Rect.New(imgX, marginTopImage, imgWidth, imgHeight));
                }
            }
        }
    },

    adjustLabelSize: function() {
        if (this._label !== null && this.frame().size.height > 0)
        {
            var labelHeight = this.frame().size.height;
            var labelWidth = this.frame().size.width - this._imageView.frame().size.width;
            this._label.setFrame(APT.Rect.New(this._imageView.frame().size.width + this._imageView.frame().origin.x + 1, 0, labelWidth, labelHeight));
        }
    },

    standardizeMarginAndPaddingFormInnerSpan: function() {
        var buttonInner = this.jQElement().children(".ui-btn-inner");
        buttonInner.css("margin-top", "0px");
        buttonInner.css("margin-bottom", "0px");
        buttonInner.css("margin-left", "0px");
        buttonInner.css("margin-right", "0px");
        buttonInner.css("padding-top", "0px");
        buttonInner.css("padding-bottom", "0px");
        buttonInner.css("padding-right", "0px");
        buttonInner.css("padding-left", "0px");
        buttonInner.css("border-top", "0px solid white");
        buttonInner.css("left", "0px");
        buttonInner.css("top", "0px");
        buttonInner.css("position", "absolute");
        var buttonText = buttonInner.children(".ui-btn-text");
        buttonText.css("margin-top", "0px");
        buttonText.css("margin-bottom", "0px");
        buttonText.css("margin-left", "0px");
        buttonText.css("margin-right", "0px");
        buttonText.css("padding-top", "0px");
        buttonText.css("padding-bottom", "0px");
        buttonText.css("padding-left", "0px");
        buttonText.css("padding-right", "0px");
        buttonText.css("border-top", "0px solid white");
        buttonText.css("left", "0px");
        buttonText.css("top", "0px");
        buttonText.css("position", "absolute");
    },

});

APT.Button.buttonWithType = function(buttonType) {
    var button = new APT.Button();
    var aObject = button.getRenderMarkup();
    aObject.find("button").attr("data-button-type", String(buttonType));
    button.setJQElement(aObject);
    return button;
};


