/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.DateComponents = Class.$define("APT.DateComponents", {
    __init__: function() {
        this._year = APT.DateComponents.Undefined;
        this._month = APT.DateComponents.Undefined;
        this._day = APT.DateComponents.Undefined;
        this._hour = APT.DateComponents.Undefined;
        this._minute = APT.DateComponents.Undefined;
        this._second = APT.DateComponents.Undefined;
        this._era = APT.DateComponents.Undefined;
        this._week = APT.DateComponents.Undefined;
        this._weekday = APT.DateComponents.Undefined;
        this._weekdayOrdinal = APT.DateComponents.Undefined;
        this._quarter = APT.DateComponents.Undefined;
        this._weekOfMonth = APT.DateComponents.Undefined;
        this._weekOfYear = APT.DateComponents.Undefined;
        this._date = undefined;
        this._calendar = undefined;
        this._unitFlags = 0;
    },

    $copy: function() {
        var copyObject = new APT.DateComponents();
        copyObject._year = this._year;
        copyObject._month = this._month;
        copyObject._day = this._day;
        copyObject._hour = this._hour;
        copyObject._minute = this._minute;
        copyObject._second = this._second;
        copyObject._era = this._era;
        copyObject._week = this._week;
        copyObject._weekday = this._weekday;
        copyObject._weekdayOrdinal = this._weekdayOrdinal;
        copyObject._quarter = this._quarter;
        copyObject._weekOfMonth = this._weekOfMonth;
        copyObject._weekOfYear = this._weekOfYear;
        copyObject._date = this._date;
        copyObject._calendar = this._calendar;
        copyObject._unitFlags = this._unitFlags;
        return copyObject;
    },

    unitFlags: function() {
        return this._unitFlags;
    },

    setUnitFlags: function(flags) {
        this._unitFlags = flags;
    },

    era: function() {
        return this._era;
    },

    setEra: function(era) {
        this._era = era;
    },

    year: function() {
        return this._year;
    },

    setYear: function(year) {
        if (this._date !== undefined)
        {
            this._date.setFullYear(year);
        }

        this._year = year;
    },

    month: function() {
        return this._month;
    },

    setMonth: function(month) {
        if (this._date !== undefined)
        {
            this._date.setMonth(month);
        }

        this._month = month;
    },

    day: function() {
        return this._day;
    },

    setDay: function(day) {
        if (this._date !== undefined)
        {
            this._date.setDate(day);
        }

        this._day = day;
    },

    hour: function() {
        return this._hour;
    },

    setHour: function(hour) {
        if (this._date !== undefined)
        {
            this._date.setHours(hour);
        }

        this._hour = hour;
    },

    minute: function() {
        return this._minute;
    },

    setMinute: function(minute) {
        if (this._date !== undefined)
        {
            this._date.setMinutes(minute);
        }

        this._minute = minute;
    },

    second: function() {
        return this._second;
    },

    setSecond: function(second) {
        if (this._date !== undefined)
        {
            this._date.setSeconds(second);
        }

        this._second = second;
    },

    setCalendar: function(calendar) {
        this._calendar = calendar;
        if (this._date === undefined)
        {
            this._date = new Date(0001, 00, 01, 0, 0, 0, 0);
            this._date.setFullYear(01);
        }
    },

    setCalendar: function(calendar, date) {
        this._calendar = calendar;
        if (this._date === undefined)
        {
            this._date = new Date(0001, 00, 01, 0, 0, 0, 0);
            this._date.setFullYear(01);
        }

        if (date !== undefined)
        {
            if (APT.CalendarUnit.Year & this._unitFlags)
            {
                this.setYear(date.getFullYear());
            }

            if (APT.CalendarUnit.Month & this._unitFlags)
            {
                this.setMonth(date.getMonth() + 1);
            }

            if (APT.CalendarUnit.Day & this._unitFlags)
            {
                this.setDay(date.getDate());
            }

            if (APT.CalendarUnit.Hour & this._unitFlags)
            {
                this.setHour(date.getHours());
            }

            if (APT.CalendarUnit.Minute & this._unitFlags)
            {
                this.setMinute(date.getMinutes());
            }

            if (APT.CalendarUnit.Second & this._unitFlags)
            {
                this.setSecond(date.getSeconds());
            }
        }
    },

    calendar: function() {
        return this._calendar;
    },

    weekday: function() {
        return this._weekday;
    },

    setWeekday: function(weekday) {
        this._weekday = weekday;
    },

    week: function() {
        return this._week;
    },

    setWeek: function(week) {
        this._week = week;
    },

    date: function() {
        return this._date;
    },

    weekdayOrdinal: function() {
        return this._weekdayOrdinal;
    },

    setWeekdayOrdinal: function(weekday) {
        this._weekdayOrdinal = weekday;
    },

    quarter: function() {
        return this._quarter;
    },

    setQuarter: function(quarter) {
        this._quarter = quarter;
    },

    weekOfMonth: function() {
        return this._weekOfMonth;
    },

    setWeekOfMonth: function(weekOfMonth) {
        this._weekOfMonth = weekOfMonth;
    },

    weekOfYear: function() {
        return this._weekOfYear;
    },

    setWeekOfYear: function(weekOfYear) {
        this._weekOfYear = weekOfYear;
    },

});

APT.DateComponents.Undefined = 2147483647;


