/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.Device = Class.$define("APT.Device", {
    __init__: function() {
        this._orientation = APT.DeviceOrientation.Unknown;
        this._notification = false;
    },

    $copy: function() {
        var copyObject = new APT.Device();
        copyObject._orientation = this._orientation;
        copyObject._notification = this._notification;
        return copyObject;
    },

    setOrientation: function(orientation) {
        if (this._notification)
        {
            this._orientation = orientation;
        }
    },

    orientation: function() {
        if (this._notification)
        {
            return this._orientation;
        }
        else
        {
            return APT.DeviceOrientation.Unknown;
        }
    },

    beginGeneratingDeviceOrientationNotifications: function() {
        var $that = this;

        $(window).on("orientationchange", function(event) {
            if (event.orientation)
            {
                if (event.orientation === "portrait")
                {
                    alert("Portrait");
                    $that.setOrientation(APT.DeviceOrientation.Portrait);
                }
                else
                {
                    if (event.orientation === "landscape")
                    {
                        alert("landscape");
                        $that.setOrientation(APT.DeviceOrientation.Landscape);
                    }
                    else
                    {
                        $that.setOrientation(APT.DeviceOrientation.Unknown);
                    }
                }
            }
            else
            {
                $that.setOrientation(APT.DeviceOrientation.Unknown);
            }

            event.stopPropagation();
        });

        this._notification = true;
    },

    endGeneratingDeviceOrientationNotifications: function() {
        $(window).off("orientationchange");
        this._notification = false;
        this._orientation = APT.DeviceOrientation.Unknown;
    },

    generatesDeviceOrientationNotifications: function() {
        return this._notification;
    },

});

APT.Device.currentDevice = function() {
    var device = new APT.Device();
    device._notification = APT.Device._notification;
    device._orientation = APT.Device._orientation;
    return device;
};


