/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.Image = Class.$define("APT.Image", {
    __init__: function() {
        this._name = null;
        this._size = null;
        this._scale = null;
    },

    $copy: function() {
        var copyObject = new APT.Image();
        copyObject._name = this._name;
        copyObject._size = this._size.$copy();
        copyObject._scale = this._scale;
        return copyObject;
    },

    name: function() {
        return this._name;
    },

    setName: function(aName) {
        this._name = aName;
    },

    drawAtPoint: function(point) {
        point = point.$copy();
        var context = APT.Window.currentGraphicsContext();
        var image = $("<img></img>");
        image.attr("src", this._name);
        if (context !== null)
        {
            context.drawImage(image[0], point.x, point.y);
        }
    },

    drawInRect: function(rect) {
        rect = rect.$copy();
        var context = APT.Window.currentGraphicsContext();
        var image = $("<img></img>");
        image.attr("src", this._name);
        if (context !== null)
        {
            context.drawImage(image[0], 0, 0, rect.size.width, rect.size.height);
        }
    },

    size: function() {
        if (this._size.width === 0 && this._size.height === 0)
        {
            var imageJQ = $("<img>");
            imageJQ.attr("src", this.name());
            imageJQ.hide();
            imageJQ.appendTo("body");
            var $that = this;
            imageJQ.on("load", function() {
                $that._size = APT.Size.updateFromCSS(jQuery(this));
                jQuery(this).detach();
            });
        }

        return this._size;
    },

    scale: function() {
        return this._scale;
    },

    imageOrientation: function() {
        return APT.ImageOrientation.Up;
    },

    initWithData: function(aData) {
        var image = new APT.Image();

        var componentsJoinedByString = "";
        for (var index = 0; index < aData.length - 1; index++)
        {
            componentsJoinedByString += aData[index].toString().valueOf();
        }

        componentsJoinedByString += aData[aData.length - 1].toString().valueOf();
        componentsJoinedByString = "data:image/png;base64," + componentsJoinedByString;
        image.setName(componentsJoinedByString);

        image._size = new APT.Size(0, 0);
        image.size();
        return image;
    },

});

APT.Image.imageNamed = function(aName) {
    var image = new APT.Image();
    image.setName(aName);
    var length = image._name.length;

    if (image._name.substring(length - 3) === "@2x")
    {
        image._scale = parseFloat(2.0);
    }
    else
    {
        image._scale = parseFloat(1.0);
    }

    image._size = new APT.Size(0, 0);
    image.size();
    return image;
};

APT.Image.imageWithContentsOfFile = function(aPath) {
    var image = new APT.Image();
    image.setName(aPath);
    image._size = new APT.Size(0, 0);
    image.size();
    return image;
};


