/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");

APT.ImageView = Class.$define("APT.ImageView", APT.View, {
    __init__: function() {
        this._image = null;
        this._animationImages = null;
        this._animationDuration = 0;
        this._animationRepeatCount = 0;
        this._interval = 0;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.ImageView();
        copyObject._image = this._image;
        copyObject._animationImages = this._animationImages;
        copyObject._animationDuration = this._animationDuration;
        copyObject._animationRepeatCount = this._animationRepeatCount;
        copyObject._interval = this._interval;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<img class='apt_imageview'></img>"));
    },

    setJQElement: function(jqElement) {
        this._jqElement = jqElement;
        this.setProperties();
        if (this.jQElement().attr("src") !== undefined)
        {
            this.setImage(APT.Image.imageNamed(this.jQElement().attr("src")));
        }
    },

    initWithImage: function(anImage) {
        this.setImage(anImage);
        return this;
    },

    image: function() {
        return this._image;
    },

    setImage: function(aImage) {
        this._image = aImage;
        if (aImage !== null)
        {
            this.jQElement().css("display", "block");
            var src = aImage.name();
            this.jQElement().attr("src", src.valueOf());
            if (aImage.size().width === 0 && aImage.size().height === 0)
            {
                this._getImageSize();
            }
            else
            {
                this.setFrame(APT.Rect.New(0, 0, aImage.size().width, aImage.size().height));
            }
        }
        else
        {
            this.jQElement().attr("src", "");
        }
    },

    animationImages: function() {
        return this._animationImages;
    },

    setAnimationImages: function(images) {
        this._animationImages = images;
    },

    animationDuration: function() {
        return this._animationDuration;
    },

    setAnimationDuration: function(duration) {
        this._animationDuration = duration;
    },

    animationRepeatCount: function() {
        return this._animationRepeatCount;
    },

    setAnimationRepeatCount: function(repeat) {
        this._animationRepeatCount = repeat;
    },

    startAnimating: function() {
        var imageview = this;
        var repeat = 0;

        this._interval = window.setInterval(function() {
            if (imageview._animationRepeatCount === 0 || imageview._animationRepeatCount > repeat)
            {
                var time = 0;
                var name = (imageview._animationImages[0]).name();

                for (var i = 0; i < imageview._animationImages.length; i++)
                {

                    window.setTimeout(function() {
                        imageview.jQElement().attr("src", name);
                        var i = 0;
                        for (i = 0; i < imageview._animationImages.length; i++)
                        {
                            if (name.valueOf() === (imageview._animationImages[i]).name().valueOf())
                            {
                                if (i < imageview._animationImages.length - 1)
                                {
                                    name = (imageview._animationImages[i + 1]).name();
                                    break;
                                }
                            }
                        }
                    }, time);
                    time += imageview._animationDuration / imageview._animationImages.length * 1000;
                }
            }
            else
            {
                imageview.stopAnimating();
            }

            repeat++;
        }, this._animationDuration * 1000);
    },

    isAnimating: function() {
        return (((this._interval === 0)) ? (false) : (true));
    },

    stopAnimating: function() {
        window.clearInterval(this._interval);
        this._interval = 0;
        this.jQElement().attr("src", "");
        this.jQElement().css("display", "none");
    },

    _getImageSize: function() {
        var imageJQ = $("<img>");
        imageJQ.attr("src", this._image.name());
        imageJQ.hide();
        imageJQ.appendTo("body");
        var $that = this;
        imageJQ.on("load", function() {
            var aRect = APT.Rect.New(0, 0, $that.frame().size.width, $that.frame().size.height);
            var mustSetFrame = isNaN($that.frame().origin.x) || isNaN($that.frame().origin.y);
            if ($that.frame().size.width === 0 && $that.frame().size.height === 0)
            {
                mustSetFrame = true;
                aRect.size = APT.Size.updateFromCSS(jQuery(this));
            }

            if (mustSetFrame)
            {
                $that.setFrame(aRect);
            }

            jQuery(this).detach();
        });
    },

});


