/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.ViewController = Class.$define("APT.ViewController");

APT.NavigationController = Class.$define("APT.NavigationController", APT.ViewController, {
    __init__: function() {
        this._stack = APT.Global.CreateStack(new Array());
        this._delegate = null;
        this._navigationBar = new APT.NavigationBar();
        this._navigationBarHidden = false;
        this._topViewController = null;
        this._visibleViewController = null;
        this._rootViewController = null;
        this._poppingStarted = false;
        this._toolbar = new APT.ToolBar();
        this._toolbarHidden = true;
        this._lastHidesBottomBarWhenPushed = true;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.NavigationController();
        copyObject._stack = this._stack;
        copyObject._delegate = this._delegate;
        copyObject._navigationBar = this._navigationBar;
        copyObject._navigationBarHidden = this._navigationBarHidden;
        copyObject._topViewController = this._topViewController;
        copyObject._visibleViewController = this._visibleViewController;
        copyObject._rootViewController = this._rootViewController;
        copyObject._poppingStarted = this._poppingStarted;
        copyObject._toolbar = this._toolbar;
        copyObject._toolbarHidden = this._toolbarHidden;
        copyObject._lastHidesBottomBarWhenPushed = this._lastHidesBottomBarWhenPushed;
        return copyObject;
    },

    initWithRootViewController: function(aViewController) {
        if (aViewController === null || aViewController === undefined)
        {
            return null;
        }
        else
        {
            this.setViewControllers([aViewController]);
            return this;
        }

        return this;
    },

    pushViewController_animated: function(aViewController, animated) {
        if (aViewController === null || aViewController === undefined || aViewController.view() === null || aViewController.view() === undefined)
        {
            return;
        }
        else
        {
            var currentViewController = this._stack.head();
            this._stack.push(aViewController);
            aViewController.setNavigationController(this);
            this._topViewController = aViewController;

            this._lastHidesBottomBarWhenPushed = this._topViewController.hidesBottomBarWhenPushed();

            this.pushNavigationItemOfController(aViewController, animated);

            if (currentViewController !== null && currentViewController !== undefined)
            {
                this.popToLeft(currentViewController, animated);
            }

            this.pushToLeft(aViewController, animated);
        }
    },

    popViewControllerAnimated: function(animated) {
        var viewController = null;
        if (this._stack.length > 1)
        {
            this._poppingStarted = true;
            this._navigationBar.popNavigationItemAnimated(animated);
            viewController = this.popController(animated);
            this.fixVisibleTitle();
        }

        return viewController;
    },

    popToRootViewControllerAnimated: function(animated) {
        return this.popToViewController_animated(this._rootViewController, animated);
    },

    popToViewController_animated: function(aViewController, animated) {
        var viewControllers = new Array();
        if (aViewController === null)
        {
            return viewControllers;
        }

        var index = this._stack.indexOf(aViewController);

        if (this._stack.length > 1 && index >= 0)
        {
            viewControllers = this._stack.splice(index + 1, this._stack.length - 1);
            var currentViewController = this._topViewController;
            this._topViewController = this._stack.head();

            this._lastHidesBottomBarWhenPushed = this._topViewController.hidesBottomBarWhenPushed();

            this.popToRight(currentViewController, animated);

            this.pushToRight(this._topViewController, animated);

            this.setNavigationItems(animated);
        }

        for (var i = 0; i < viewControllers.length; i++)
        {
            var vc = viewControllers[i];
            vc.setNavigationController(null);
        }

        return viewControllers;
    },

    setViewControllers: function(viewControllers) {
        if (viewControllers !== null && viewControllers.length > 0)
        {
            this._stack = APT.Global.CreateStack(viewControllers);
            this._rootViewController = this._stack[0];
            this._topViewController = this._stack[this._stack.length - 1];
            for (var i = 0; i < this._stack.length; i++)
            {
                var vc = this._stack[i];
                vc.setNavigationController(this);
            }

            this._toolbar.setItems(this._topViewController.toolbarItems());
        }

        this.resize(true);
    },

    loadView: function() {
        this.setView(new APT.View());
        this.view().setFrame(APT.Screen.MainScreen().bounds());
        this.view().jQElement().addClass("apt_navigationView");
        this.view().addSubview(this._navigationBar);
    },

    viewWillAppear: function(animated) {
        this._topViewController = this._stack.head();
        if (this._topViewController !== null)
        {
            this.view().addSubviewWithoutNotifying(this._topViewController.view(), 0);
            this.setNavigationItems(false);
            this._topViewController.view().viewWillAppear(animated);
        }

        this.resize(false);
    },

    viewControllers: function() {
        return this._stack.apt_copy();
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
    },

    delegate: function() {
        return this._delegate;
    },

    topViewController: function() {
        return this._topViewController;
    },

    setTabBarController: function(aTabBarController) {
        this.$super(aTabBarController);
        for (var i = 0; i < this._stack.length; i++)
        {
            (this._stack[i]).setTabBarController(this.tabBarController());
        }
    },

    pushNavigationItemOfController: function(aViewController, animated) {
        var backButtonItem = new APT.BarButtonItem();
        backButtonItem.setTarget(this);
        backButtonItem.setTitle("Back");
        backButtonItem.setAction("popViewControllerAnimated");
        aViewController.navigationItem().setBackBarButtonItem(backButtonItem);
        this._navigationBar.pushNavigationItem_animated(aViewController.navigationItem(), animated);
        this.fixVisibleTitle();
    },

    popController: function(animated) {
        var viewController = this._stack.pop();

        this._topViewController = this._stack.head();

        this._lastHidesBottomBarWhenPushed = this._topViewController.hidesBottomBarWhenPushed();

        this.popToRight(viewController, animated);

        this.pushToRight(this._topViewController, animated);

        return viewController;
    },

    getNavigationItems: function() {
        var items = new Array();
        for (var i = 0; i < this._stack.length; i++)
        {
            var vc = this._stack[i];
            items.push(vc.navigationItem());
        }

        return items;
    },

    setNavigationItems: function(animated) {
        this._navigationBar.setItems_animated(this.getNavigationItems(), animated);
        this.fixVisibleTitle();
    },

    fixVisibleTitle: function() {
        this._navigationBar.displayCenterItems();
    },

    bar: function() {
        return this._navigationBar;
    },

    rootViewController: function() {
        return this._rootViewController;
    },

    toolbar: function() {
        return this._toolbar;
    },

    isToolbarHidden: function() {
        return this._toolbarHidden || this._lastHidesBottomBarWhenPushed;
    },

    setToolbarHidden: function(toolbarHidden) {
        this.setToolbarHidden_animated(toolbarHidden, false);
    },

    setToolbarHidden_animated: function(toolbarHidden, animated) {
        if (this._view !== null && this.view().isInDom() && this.isToolbarHidden() !== toolbarHidden)
        {
            var contentX = 0;
            var contentY = 0;
            var contentWidth = parseInt(this.view().frame().size.width);
            var contentHeight = parseInt(this.view().frame().size.height);

            var toolbarY = this.view().frame().size.height - APT.NavigationController._toolbarHeight;

            if (!this._navigationBarHidden)
            {
                contentY = APT.NavigationController._navigationBarHeight;
                contentHeight = contentHeight - APT.NavigationController._navigationBarHeight;
            }

            if (animated)
            {
                if (toolbarHidden)
                {
                    this.removeToolbarAnimated(contentHeight, toolbarY);
                }
                else
                {
                    contentHeight = contentHeight - APT.NavigationController._toolbarHeight;
                    this.addToolbarAnimated(contentHeight, toolbarY);
                }
            }
            else
            {
                if (toolbarHidden)
                {
                    this._navigationBar.removeFromSuperview();
                }
                else
                {
                    if (!this._toolbar.isInDom())
                    {
                        this.view().addSubview(this._toolbar);
                    }

                    contentHeight = contentHeight - APT.NavigationController._toolbarHeight;
                }

                this._topViewController.view().setFrame(APT.Rect.New(contentX, contentY, contentWidth, contentHeight));
                this._toolbar.setFrame(APT.Rect.New(0, toolbarY, this.view().frame().size.width, APT.NavigationController._toolbarHeight));
            }
        }

        this._toolbarHidden = toolbarHidden;
        this._lastHidesBottomBarWhenPushed = toolbarHidden;
    },

    isNavigationBarHidden: function() {
        return this._navigationBarHidden;
    },

    setNavigationBarHidden: function(navigationBarHidden) {
        this.setNavigationBarHidden_animated(navigationBarHidden, false);
    },

    setNavigationBarHidden_animated: function(navigationBarHidden, animated) {
        if (this._view !== null && this.view().isInDom() && this._navigationBarHidden !== navigationBarHidden)
        {
            var contentX = 0;
            var contentY = 0;
            var contentWidth = parseInt(this.view().frame().size.width);
            var contentHeight = parseInt(this.view().frame().size.height);

            var toolbarY = this.view().frame().size.height - APT.NavigationController._toolbarHeight;

            if (!this.isToolbarHidden())
            {
                contentHeight = contentHeight - APT.NavigationController._toolbarHeight;
            }

            if (animated)
            {
                if (navigationBarHidden)
                {
                    this.removeNavigationBarAnimated(contentHeight, contentY);
                }
                else
                {
                    contentY = APT.NavigationController._navigationBarHeight;
                    contentHeight = contentHeight - APT.NavigationController._navigationBarHeight;
                    this.addNavigationBarAnimated(contentHeight, contentY);
                }
            }
            else
            {
                if (navigationBarHidden)
                {
                    this._navigationBar.removeFromSuperview();
                }
                else
                {
                    if (!this._navigationBar.isInDom())
                    {
                        this.view().addSubview(this._navigationBar);
                    }

                    contentY = APT.NavigationController._navigationBarHeight;
                    contentHeight = contentHeight - APT.NavigationController._navigationBarHeight;
                }

                this._topViewController.view().setFrame(APT.Rect.New(contentX, contentY, contentWidth, contentHeight));
                this._navigationBar.setFrame(APT.Rect.New(0, 0, this.view().frame().size.width, APT.NavigationController._navigationBarHeight));
            }
        }

        this._navigationBarHidden = navigationBarHidden;
    },

    removeNavigationBarAnimated: function(contentHeight, contentY) {
        this._topViewController.view().jQElement().animate({"top" : new Number(contentY).toString(), "height" : new Number(contentHeight).toString() + "px"}, "fast");

        var thisNavigationBar = this._navigationBar;

        this._navigationBar.jQElement().animate({"top" : "-" + new Number(APT.NavigationController._navigationBarHeight).toString() + "px"}, "fast", function() {
            thisNavigationBar.removeFromSuperview();
        });
    },

    addNavigationBarAnimated: function(contentHeight, contentY) {
        this._navigationBar.setFrame(APT.Rect.New(0, -APT.NavigationController._navigationBarHeight, this.view().frame().size.width, APT.NavigationController._navigationBarHeight));
        this.view().addSubview(this._navigationBar);

        this._topViewController.view().jQElement().animate({"top" : new Number(contentY).toString() + "px", "height" : new Number(contentHeight).toString() + "px"}, "fast");

        this._navigationBar.jQElement().animate({"top" : new Number(0).toString() + "px"}, "fast");
    },

    removeToolbarAnimated: function(contentHeight, toolbarY) {
        this._topViewController.view().jQElement().animate({"height" : new Number(contentHeight).toString() + "px"}, "fast");
        var thisToolbar = this._toolbar;

        this._toolbar.jQElement().animate({"top" : new Number(toolbarY + APT.NavigationController._toolbarHeight).toString() + "px"}, "fast", function() {
            thisToolbar.removeFromSuperview();
        });
    },

    addToolbarAnimated: function(contentHeight, toolbarY) {
        this._toolbar.setFrame(APT.Rect.New(0, toolbarY + APT.NavigationController._toolbarHeight, this.view().frame().size.width, APT.NavigationController._toolbarHeight));
        this.view().addSubview(this._toolbar);

        this._topViewController.view().jQElement().animate({"height" : new Number(contentHeight).toString() + "px"}, "fast");

        this._toolbar.jQElement().animate({"top" : new Number(toolbarY).toString() + "px"}, "fast");
    },

    resize: function(checkViewInDom) {
        var size = new APT.Size(0, 0);
        if (this.view().frame() !== null)
        {
            size = this.view().frame().size.$copy();
        }

        var navigationBarRect = APT.Rect.New(0, 0, size.width, APT.NavigationController._navigationBarHeight);
        var toolbarRect = APT.Rect.New(0, size.height - APT.NavigationController._toolbarHeight, size.width, APT.NavigationController._toolbarHeight);
        this._navigationBar.setFrame(navigationBarRect);
        this._toolbar.setFrame(toolbarRect);
        this._topViewController = this._stack.head();

        if (!checkViewInDom || this.view().isInDom())
        {
            var contentY = 0;
            var contentX = 0;
            var contentWidth = parseInt(size.width);
            var contentHeight = parseInt(size.height);

            if (this._navigationBarHidden)
            {
                this._navigationBar.removeFromSuperview();
            }
            else
            {
                if (!this._navigationBar.isInDom())
                {
                    this.view().addSubview(this._navigationBar);
                }

                contentY = APT.NavigationController._navigationBarHeight;
                contentHeight = contentHeight - APT.NavigationController._navigationBarHeight;
            }

            this._toolbar.setItems(this._topViewController.toolbarItems());

            if (this.isToolbarHidden())
            {
                this._toolbar.removeFromSuperview();
            }
            else
            {
                if (!this._toolbar.isInDom())
                {
                    this.view().addSubview(this._toolbar);
                }

                contentHeight = contentHeight - APT.NavigationController._toolbarHeight;
            }

            this._topViewController.view().setFrame(APT.Rect.New(contentX, contentY, contentWidth, contentHeight));
        }
    },

    popToLeft: function(aViewController, animated) {
        aViewController.view().viewWillDisappear(animated);
        if (animated)
        {

            aViewController.view().jQElement().animate({"left" : "-" + this.view().jQElement().css("width")}, "fast", function() {
                aViewController.view().removeFromSuperviewWithoutNotifying();
            });

            var thisToolbar = this._toolbar;
            if (this._toolbar.isInDom() === true && this.isToolbarHidden() === true)
            {
                var thisTopViewController = this._topViewController;

                this._toolbar.jQElement().animate({"left" : "-" + this.view().jQElement().css("width")}, "fast", function() {
                    thisToolbar.setItems(thisTopViewController.toolbarItems());
                    thisToolbar.removeFromSuperview();
                });
            }
            else
            {
                thisToolbar.setItems(this._topViewController.toolbarItems());
            }

            aViewController.view().viewDidDisappear(animated);
        }
        else
        {
            this._toolbar.setItems(this._topViewController.toolbarItems());
            aViewController.view().removeFromSuperviewWithoutNotifying();
            aViewController.view().viewDidDisappear(animated);
        }
    },

    popToRight: function(aViewController, animated) {
        aViewController.view().viewWillDisappear(animated);
        if (animated)
        {

            aViewController.view().jQElement().animate({"left" : this.view().jQElement().css("width")}, "fast", function() {
                aViewController.view().removeFromSuperviewWithoutNotifying();
                aViewController.setNavigationController(null);
            });

            if (this._toolbar.isInDom() === false && this.isToolbarHidden() === false)
            {
                this._toolbar.jQElement().css("left", new Number(-this.view().frame().size.width).toString() + "px");
                this._toolbar.setItems(this._topViewController.toolbarItems());
                this.view().addSubview(this._toolbar);

                this._toolbar.jQElement().animate({"left" : "0px"}, "fast");
            }
            else
            {
                this._toolbar.setItems(this._topViewController.toolbarItems());
            }

            aViewController.view().viewDidDisappear(animated);
        }
        else
        {
            aViewController.view().removeFromSuperviewWithoutNotifying();
            this._toolbar.setItems(this._topViewController.toolbarItems());
            aViewController.view().viewDidDisappear(animated);
            aViewController.setNavigationController(null);
        }
    },

    pushToLeft: function(aViewController, animated) {
        if (this._delegate !== null && this._delegate["navigationController_willShowViewController_animated"] instanceof Function)
        {
            this._delegate.navigationController_willShowViewController_animated(this, aViewController, animated);
        }

        aViewController.view().viewWillAppear(animated);

        var contentY = 0;
        var contentX = 0;
        var contentWidth = parseInt(this.view().frame().size.width);
        var contentHeight = parseInt(this.view().frame().size.height);

        if (this._navigationBarHidden === false)
        {
            contentY = APT.NavigationController._navigationBarHeight;
            contentHeight = contentHeight - APT.NavigationController._navigationBarHeight;
        }

        if (this.isToolbarHidden() === false)
        {
            contentHeight = contentHeight - APT.NavigationController._toolbarHeight;
        }

        if (animated)
        {
            aViewController.view().setFrame(APT.Rect.New(contentWidth, contentY, contentWidth, contentHeight));
            aViewController.view().jQElement().show();
            if (aViewController.view().canvas() !== null)
            {
                this.view().addSubviewWithoutNotifying(aViewController.view(), 1);
            }
            else
            {
                this.view().addSubviewWithoutNotifying(aViewController.view(), 0);
            }

            var thisNavigationController = this;

            aViewController.view().jQElement().animate({"left" : new Number(contentX).toString() + "px"}, "fast", function() {
                if (thisNavigationController._delegate !== null && thisNavigationController._delegate["navigationController_didShowViewController_animated"] instanceof Function)
                {
                    thisNavigationController._delegate.navigationController_didShowViewController_animated(thisNavigationController, aViewController, animated);
                }

                aViewController.view().viewDidAppear(animated);
            });
        }
        else
        {
            aViewController.view().setFrame(APT.Rect.New(contentX, contentY, contentWidth, contentHeight));
            aViewController.view().jQElement().show();
            this.view().addSubviewWithoutNotifying(aViewController.view(), 0);
            if (this._delegate !== null && this._delegate["navigationController_didShowViewController_animated"] instanceof Function)
            {
                this._delegate.navigationController_didShowViewController_animated(this, aViewController, animated);
            }

            aViewController.view().viewDidAppear(animated);
        }
    },

    pushToRight: function(aViewController, animated) {
        if (this._delegate !== null && this._delegate["navigationController_willShowViewController_animated"] instanceof Function)
        {
            this._delegate.navigationController_willShowViewController_animated(this, aViewController, animated);
        }

        aViewController.view().viewWillAppear(animated);

        var contentY = 0;
        var contentX = 0;
        var contentWidth = parseInt(this.view().frame().size.width);
        var contentHeight = parseInt(this.view().frame().size.height);

        if (this._navigationBarHidden === false)
        {
            contentY = APT.NavigationController._navigationBarHeight;
            contentHeight = contentHeight - APT.NavigationController._navigationBarHeight;
        }

        if (this.isToolbarHidden() === false)
        {
            contentHeight = contentHeight - APT.NavigationController._toolbarHeight;
        }

        if (animated)
        {
            aViewController.view().setFrame(APT.Rect.New(-contentWidth, contentY, contentWidth, contentHeight));
            aViewController.view().jQElement().show();
            this.view().addSubviewWithoutNotifying(aViewController.view(), 1);
            var thisNavigationController = this;

            aViewController.view().jQElement().animate({"left" : new Number(contentX).toString() + "px"}, "fast", function() {
                thisNavigationController._toolbar.setItems(aViewController.toolbarItems());
                if (thisNavigationController._delegate !== null && thisNavigationController._delegate["navigationController_didShowViewController_animated"] instanceof Function)
                {
                    thisNavigationController._delegate.navigationController_didShowViewController_animated(thisNavigationController, aViewController, animated);
                }

                aViewController.view().viewDidAppear(animated);
            });
        }
        else
        {
            aViewController.view().setFrame(APT.Rect.New(contentX, contentY, contentWidth, contentHeight));
            aViewController.view().jQElement().show();
            aViewController.view().jQElement().css("left", "0px");
            this.view().addSubviewWithoutNotifying(aViewController.view(), 1);
            this._toolbar.setItems(this._topViewController.toolbarItems());
            if (this._delegate !== null && this._delegate["navigationController_didShowViewController_animated"] instanceof Function)
            {
                this._delegate.navigationController_didShowViewController_animated(this, aViewController, animated);
            }

            aViewController.view().viewDidAppear(animated);
        }
    },

});

APT.NavigationController._navigationBarHeight = 44;

APT.NavigationController._toolbarHeight = 44;


