/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");
APT.Control = Class.$define("APT.Control", APT.View);

APT.PageControl = Class.$define("APT.PageControl", APT.Control, {
    __init__: function() {
        this._currentPage = 0;
        this._defersCurrentPageDisplay = false;
        this._hidesForSinglePage = false;
        this._numberOfPages = 0;
        this._userInteractionEnabled = true;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.PageControl();
        copyObject._currentPage = this._currentPage;
        copyObject._defersCurrentPageDisplay = this._defersCurrentPageDisplay;
        copyObject._hidesForSinglePage = this._hidesForSinglePage;
        copyObject._numberOfPages = this._numberOfPages;
        copyObject._userInteractionEnabled = this._userInteractionEnabled;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<div data-role='controlgroup' data-type='horizontal'></div>"));
    },

    setJQElement: function(jqelement) {
        var text = jqelement.text();
        jqelement.text("");
        this._jqElement = jqelement.controlgroup();
        if (text.length > 0)
        {
            this.setNumberOfPages(parseInt(text));
        }

        this.jQElement().css("margin", 0);

        this.setProperties();
        var padding = (this.frame().size.height - 16) / 2;
        this.jQElement().find(".ui-controlgroup-controls").css("padding", padding);
    },

    currentPage: function() {
        return this._currentPage;
    },

    setCurrentPage: function(newPage) {
        this._currentPage = newPage;
        this.updateCurrentPageDisplay();
    },

    defersCurrentPageDisplay: function() {
        return this._defersCurrentPageDisplay;
    },

    setDefersCurrentPageDisplay: function(value) {
        this._defersCurrentPageDisplay = value;
    },

    hidesForSinglePage: function() {
        return this._hidesForSinglePage;
    },

    setHidesForSinglePage: function(value) {
        this._hidesForSinglePage = value;
        if (value && this.numberOfPages() === 1)
        {
            this.setHidden(true);
        }
    },

    numberOfPages: function() {
        return this._numberOfPages;
    },

    setNumberOfPages: function(pageCount) {
        this._numberOfPages = pageCount;
        var pageControl = this;
        var srcImg;

        this.jQElement().find(".ui-controlgroup-controls").children().detach();

        for (var i = 0; i < pageCount; i++)
        {
            srcImg = "";
            if (i === 0)
            {
                srcImg = "apt_pageControl_selectedPage";
            }
            else
            {
                srcImg = "apt_pageControl_notSelectedPage";
            }

            var radio = $(APT.Global.toStaticHTML("<div />"));
            radio.attr("class", srcImg);
            radio.attr("value", new String(i));
            radio.attr("id", "div" + new String(i));
            var eventHandler = function(event) {
                event.stopPropagation();
                if (pageControl._userInteractionEnabled)
                {
                    var val = parseInt(jQuery(this).index());
                    var current = pageControl.currentPage();
                    if (current !== val)
                    {
                        if ((current - 1 === val || current + 1 === val) && current !== val)
                        {
                            if (!pageControl.defersCurrentPageDisplay())
                            {
                                jQuery(this).parent().children().attr("class", "apt_pageControl_notSelectedPage");
                                jQuery(this).attr("class", "apt_pageControl_selectedPage");
                            }

                            pageControl.setCurrentPage(val);
                            pageControl.sendActionsForControlEvents(APT.ControlEvent.ValueChanged);
                        }
                        else
                        {
                            if (current < val - 1)
                            {
                                current = current + 1;
                            }
                            else
                            {
                                if (current > val)
                                {
                                    current = current - 1;
                                }
                            }

                            event.preventDefault();
                            var buttons = jQuery(this).parent().children();
                            event.currentTarget = buttons[current];
                            event.target = buttons[current];
                            jQuery(buttons[current]).trigger("vmousedown", event);
                        }
                    }
                }
            };

            radio.on("vmousedown", eventHandler);
            this.jQElement().find(".ui-controlgroup-controls").append(radio);
        }

        this.jQElement().find(".ui-controlgroup-controls").attr("style", "margin-right: auto; margin-left: auto; display:block;");
        this.jQElement().find(".ui-controlgroup-controls").css("width", 20 * pageCount);
    },

    sizeForNumberOfPages: function(pageCount) {
        var width = 6 * this._numberOfPages + 10 * (this._numberOfPages - 1);
        return new APT.Size(width, 36);
    },

    updateCurrentPageDisplay: function() {
        this.jQElement().find(".ui-controlgroup-controls").children().attr("class", "apt_pageControl_notSelectedPage");
        $(this.jQElement().find(".ui-controlgroup-controls").children()[this.currentPage()]).attr("class", "apt_pageControl_selectedPage");
    },

    setUserInteractionEnabled: function(enabled) {
        this._userInteractionEnabled = enabled;
    },

    isUserInteractionEnabled: function() {
        return this._userInteractionEnabled;
    },

});


