/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");

APT.PickerView = Class.$define("APT.PickerView", APT.View, {
    __init__: function() {
        this._parameters = new Array();
        this._picker = new Array();
        this._userInteractionEnabled = true;
        this._delegate = null;
        this._dataSource = null;
        this.$super();
        this._parameters["preset"] = new String("select");
        this._parameters["theme"] = new String("default");
        this._parameters["display"] = new String("inline");
        this._parameters["mode"] = new String("scroller");
        this._parameters["inputClass"] = new String("i-txt");
    },

    $copy: function() {
        var copyObject = new APT.PickerView();
        copyObject._parameters = this._parameters;
        copyObject._picker = this._picker;
        copyObject._userInteractionEnabled = this._userInteractionEnabled;
        copyObject._delegate = this._delegate;
        copyObject._dataSource = this._dataSource;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<div><div style='float:left;'><select style=\"display:none\"></div></div>"));
    },

    setJQElement: function(jqelement) {
        this._jqElement = jqelement.trigger("create");
    },

    numberOfComponents: function() {
        return this._dataSource.numberOfComponentsInPickerView(this);
    },

    numberOfRowsInComponent: function(component) {
        return this._dataSource.pickerView_numberOfRowsInComponent(this, component);
    },

    rowSizeForComponent: function(component) {
        var height = 0;
        var width = 0;
        if (this._delegate["pickerView_rowHeightForComponent"] instanceof Function)
        {
            height = parseFloat(this._delegate.pickerView_rowHeightForComponent(this, component));
            this._parameters["height"] = new String(height);
        }
        else
        {
            height = 40;
        }

        if (this._delegate["pickerView_widthForComponent"] instanceof Function)
        {
            width = parseFloat(this._delegate.pickerView_widthForComponent(this, component));
            this._parameters["width"] = new String(width);
        }
        else
        {
            width = 80;
        }

        return new APT.Size(height, width);
    },

    addTarget_action_forControlEvents: function(targetObject, selector, controlEvents) {
        var eventHandler = function(event) {
            targetObject[selector.valueOf()](event);
        };

        this._parameters["onChange"] = eventHandler;
        this.jQElement().scroller("destroy");
        var eventName = "." + selector;
        (this.jQElement().scroller(this._parameters)).on("change" + eventName, eventHandler);
    },

    removeTarget_action_forControlEvents: function(targetObject, selector, controlEvents) {
        this._parameters["onChange"] = null;
        this.jQElement().scroller("destroy");
        var eventName = "." + selector;
        (this.jQElement().scroller(this._parameters)).off("change" + eventName);
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
        if (this._delegate !== null)
        {
            var pickerView = this;
            var $that = this;
            var eventHandler = function() {
                var aux = jQuery(this).scroller("getValue");
                var comp = 0;

                for (var i = 0; i < pickerView._picker.length; i++)
                {
                    if (this === pickerView._picker[i].find("select")[0])
                    {
                        comp = i;
                        break;
                    }
                }

                aux = aux[0].replace("_", "");
                if ($that._delegate !== null && $that._delegate["pickerView_didSelectRow_inComponent"] instanceof Function)
                {
                    pickerView._delegate.pickerView_didSelectRow_inComponent(pickerView, parseInt(aux), comp);
                }
            };

            this._parameters["onChange"] = eventHandler;
            this.jQElement().find("select").scroller("destroy");
            (this.jQElement().find("select").scroller(this._parameters)).on("change", eventHandler);
        }
    },

    delegate: function() {
        return this._delegate;
    },

    selectRowInComponentAnimated: function(row, component, animated) {
        var values = ["_" + row];
        if (animated)
        {
            $(this.jQElement().find("select")[component]).scroller("setValue", values, false, 1);
        }
        else
        {
            $(this.jQElement().find("select")[component]).scroller("setValue", values);
        }
    },

    selectedRowInComponent: function(component) {
        var aux = $(this.jQElement().find("select")[component]).scroller("getValue");
        return parseInt((aux[0]).replace("_", ""));
    },

    setDataSource: function(aDataSource) {
        this._dataSource = aDataSource;
    },

    dataSource: function() {
        return this._dataSource;
    },

    viewDidAppear: function(animated) {
        this.reloadAllComponents();
        this.$super(animated);
    },

    reloadAllComponents: function() {
        var title;

        if (this._dataSource !== null && this._dataSource["pickerView_numberOfRowsInComponent"] instanceof Function)
        {
            this.jQElement().css("width", "100%");
            this.jQElement().scroller("destroy");
            var cantComponent = this._dataSource.numberOfComponentsInPickerView(this);
            for (var i = 0; i < cantComponent; i++)
            {
                if (i === 0)
                {
                    $(this.jQElement().children(1)).remove();
                }

                this._picker.push(this.getRenderMarkup().children(1));
                this.jQElement().append(this._picker[i]);

                if (this._dataSource["pickerView_numberOfRowsInComponent"] instanceof Function)
                {
                    var cantRows = this._dataSource.pickerView_numberOfRowsInComponent(this, i);
                    if (this._delegate !== null)
                    {
                        for (var j = 0; j < cantRows; j++)
                        {
                            if (this._delegate["pickerView_titleForRow_forComponent"] instanceof Function)
                            {
                                title = this._delegate.pickerView_titleForRow_forComponent(this, j, i);
                                var jqPicker = $("<option value=\"" + j + "\">" + title + "</option>");
                                $(this.jQElement().find("select")[i]).append(jqPicker);
                            }
                        }
                    }
                }

                $(this.jQElement().find("select")[i]).scroller(this._parameters);
                this.updateCss();
            }
        }
    },

    setUserInteractionEnabled: function(enabled) {
        this._userInteractionEnabled = !enabled;
        for (var i = 0; i < this._picker.length; i++)
        {
            $(this._picker[i]).find("select").scroller("option", "readonly", this._userInteractionEnabled);
        }

        this.updateCss();
    },

    isUserInteractionEnabled: function() {
        return !this._userInteractionEnabled;
    },

    updateCss: function() {
        this.jQElement().find("input").css("display", "none");
        this.jQElement().find(".dw").css("background", "transparent");
        this.jQElement().find(".dw").css("margin", "0px");
        this.jQElement().find(".dw").css("padding", "0px");
        this.jQElement().find(".dwl").css("display", "none");
    },

});


