/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.Rect = Class.$define("APT.Rect", {
    __init__: function(origin, size) {
        this.origin = new APT.Point();
        this.size = new APT.Size();
        if (arguments.length > 0)
        {
            this.origin = origin.$copy();
            this.size = size.$copy();
        }
    },

    $copy: function() {
        var copyObject = new APT.Rect();
        copyObject.origin = this.origin.$copy();
        copyObject.size = this.size.$copy();
        return copyObject;
    },

    divide: function(amount, edge) {
        var slice = APT.Rect.New(this.origin.x, this.origin.y, this.size.width, this.size.height);
        switch(edge)
        {
            case APT.RectEdge.MinX:
                slice.size.width = amount;
                break;
            case APT.RectEdge.MinY:
                slice.size.height = amount;
                break;
            case APT.RectEdge.MaxX:
                slice.origin.x += this.size.width - amount;
                slice.size.width = amount;
                break;
            case APT.RectEdge.MaxY:
                slice.origin.y += this.size.height - amount;
                slice.size.height = amount;
                break;
        }

        return slice;
    },

    remainder: function(amount, edge) {
        var remainder = APT.Rect.New(this.origin.x, this.origin.y, this.size.width, this.size.height);
        switch(edge)
        {
            case APT.RectEdge.MinX:
                remainder.origin.x += amount;
                remainder.size.width -= amount;
                break;
            case APT.RectEdge.MinY:
                remainder.origin.y += amount;
                remainder.size.height -= amount;
                break;
            case APT.RectEdge.MaxX:
                remainder.size.width -= amount;
                break;
            case APT.RectEdge.MaxY:
                remainder.size.height -= amount;
                break;
        }

        return remainder;
    },

});

APT.Rect.New = function(x, y, width, height) {
    return new APT.Rect(new APT.Point(x, y), new APT.Size(width, height));
};

APT.Rect.GetMidX = function(aRect) {
    aRect = aRect.$copy();
    return APT.Rect.GetMinX(aRect) + APT.Rect.Standardize(aRect).size.width / 2;
};

APT.Rect.GetMidY = function(aRect) {
    aRect = aRect.$copy();
    return APT.Rect.GetMinY(aRect) + APT.Rect.Standardize(aRect).size.height / 2;
};

APT.Rect.GetWidth = function(aRect) {
    aRect = aRect.$copy();
    return aRect.size.width;
};

APT.Rect.GetHeight = function(aRect) {
    aRect = aRect.$copy();
    return aRect.size.height;
};

APT.Rect.GetMaxX = function(aRect) {
    aRect = aRect.$copy();
    return APT.Rect.Standardize(aRect).origin.x + APT.Rect.Standardize(aRect).size.width;
};

APT.Rect.GetMaxY = function(aRect) {
    aRect = aRect.$copy();
    return APT.Rect.Standardize(aRect).origin.y + APT.Rect.Standardize(aRect).size.height;
};

APT.Rect.GetMinX = function(aRect) {
    aRect = aRect.$copy();
    return APT.Rect.Standardize(aRect).origin.x;
};

APT.Rect.GetMinY = function(aRect) {
    aRect = aRect.$copy();
    return APT.Rect.Standardize(aRect).origin.y;
};

APT.Rect.Union = function(aRect1, aRect2) {
    aRect1 = aRect1.$copy();
    aRect2 = aRect2.$copy();
    var x1 = APT.Rect.GetMinX(aRect1);
    var x2 = APT.Rect.GetMinX(aRect2);

    var x = (((x1 > x2)) ? (x2) : (x1));

    var y1 = APT.Rect.GetMinY(aRect1);
    var y2 = APT.Rect.GetMinY(aRect2);

    var y = (((y1 > y2)) ? (y2) : (y1));

    var w1 = APT.Rect.GetMaxX(aRect1);
    var w2 = APT.Rect.GetMaxX(aRect2);

    var w = (((w2 > w1)) ? (w2) : (w1));
    w -= x;

    var h1 = APT.Rect.GetMaxY(aRect1);
    var h2 = APT.Rect.GetMaxY(aRect2);

    var h = (((h2 > h1)) ? (h2) : (h1));
    h -= y;

    return new APT.Rect(new APT.Point(x, y), new APT.Size(w, h));
};

APT.Rect.Offset = function(aRect1, dx, dy) {
    aRect1 = aRect1.$copy();
    return new APT.Rect(new APT.Point(aRect1.origin.x + dx, aRect1.origin.y + dy), aRect1.size);
};

APT.Rect.EqualToRect = function(aRect, aRect2) {
    aRect = aRect.$copy();
    aRect2 = aRect2.$copy();
    return APT.Size.EqualToSize(aRect.size, aRect2.size) && APT.Point.EqualToPoint(aRect.origin, aRect2.origin);
};

APT.Rect.ContainsPoint = function(aRect, aPoint) {
    aRect = aRect.$copy();
    aPoint = aPoint.$copy();
    if (aPoint.x >= APT.Rect.GetMinX(aRect) && aPoint.x <= APT.Rect.GetMaxX(aRect) && aPoint.y >= APT.Rect.GetMinY(aRect) && aPoint.y <= APT.Rect.GetMaxY(aRect))
    {
        return true;
    }

    return false;
};

APT.Rect.ContainsRect = function(aRect, aRect2) {
    aRect = aRect.$copy();
    aRect2 = aRect2.$copy();
    if (APT.Rect.ContainsPoint(aRect, aRect2.origin) && APT.Rect.GetMaxX(aRect) >= APT.Rect.GetMaxX(aRect2) && APT.Rect.GetMaxY(aRect) >= APT.Rect.GetMaxY(aRect2))
    {
        return true;
    }

    return false;
};

APT.Rect.IsEmpty = function(rect) {
    rect = rect.$copy();
    return APT.Rect.IsNull(rect) || rect.size.width === 0 || rect.size.height === 0;
};

APT.Rect.IsNull = function(rect) {
    rect = rect.$copy();
    return APT.Rect.EqualToRect(APT.Rect.Null(), rect);
};

APT.Rect.IsInfinite = function(rect) {
    rect = rect.$copy();
    return rect.size.width === Number.POSITIVE_INFINITY || rect.size.height === Number.POSITIVE_INFINITY || rect.size.width === Number.NEGATIVE_INFINITY || rect.size.height === Number.NEGATIVE_INFINITY;
};

APT.Rect.Intersection = function(r1, r2) {
    r1 = r1.$copy();
    r2 = r2.$copy();
    var result = APT.Rect.Null;
    if (APT.Rect.IntersectsRect(r1, r2))
    {
        result = APT.Rect.New(0, 0, 0, 0);
        result.origin.x = Math.max(r1.origin.x, r2.origin.x);
        result.origin.y = Math.max(r1.origin.y, r2.origin.y);
        result.size.width = Math.min(r1.origin.x + r1.size.width, r2.origin.x + r2.size.width) - result.origin.x;
        result.size.height = Math.min(r1.origin.y + r1.size.height, r2.origin.y + r2.size.height) - result.origin.y;
    }

    return result;
};

APT.Rect.IntersectsRect = function(rect1, rect2) {
    rect1 = rect1.$copy();
    rect2 = rect2.$copy();
    var xOverlap = rect1.origin.x > rect2.origin.x && rect1.origin.x < rect2.origin.x + rect2.size.width || rect2.origin.x > rect1.origin.x && rect2.origin.x < rect1.origin.x + rect1.size.width;
    var yOverlap = rect1.origin.y > rect2.origin.y && rect1.origin.y < rect2.origin.y + rect2.size.height || rect2.origin.y > rect1.origin.y && rect2.origin.y < rect1.origin.y + rect2.size.height;
    return xOverlap && yOverlap;
};

APT.Rect.Standardize = function(rect) {
    rect = rect.$copy();
    var result = APT.Rect.New(0, 0, 0, 0);
    result.origin.x = Math.min(rect.origin.x, rect.origin.x + rect.size.width);
    result.origin.y = Math.min(rect.origin.y, rect.origin.y + rect.size.height);
    result.size.width = Math.max(rect.origin.x, rect.origin.x + rect.size.width) - result.origin.x;
    result.size.height = Math.max(rect.origin.y, rect.origin.y + rect.size.height) - result.origin.y;
    return result;
};

APT.Rect.Integral = function(rect) {
    rect = rect.$copy();
    var standardizeRect = APT.Rect.Standardize(rect);
    var result = APT.Rect.New(0, 0, 0, 0);
    result.origin.x = (((standardizeRect.origin.x % 1 === 0)) ? (standardizeRect.origin.x) : ((((standardizeRect.origin.x >= 0)) ? (parseInt(standardizeRect.origin.x)) : ((parseInt(standardizeRect.origin.x) - 1)))));
    result.origin.y = (((standardizeRect.origin.y % 1 === 0)) ? (standardizeRect.origin.y) : ((((standardizeRect.origin.y >= 0)) ? (parseInt(standardizeRect.origin.y)) : ((parseInt(standardizeRect.origin.y) - 1)))));
    var resultWidth = standardizeRect.origin.x + standardizeRect.size.width - result.origin.x;
    var resultHeight = standardizeRect.origin.y + standardizeRect.size.height - result.origin.y;
    result.size.width = (((resultWidth % 1 === 0)) ? (resultWidth) : (parseInt(resultWidth + 1)));
    result.size.height = (((resultHeight % 1 === 0)) ? (resultHeight) : (parseInt(resultHeight + 1)));
    return result;
};

APT.Rect.Inset = function(rect, dx, dy) {
    rect = rect.$copy();
    return APT.Rect.New(rect.origin.x + dx, rect.origin.y + dy, rect.size.width - dx * 2, rect.size.height - dy * 2);
};

APT.Rect.Null = function() {
    return APT.Rect.New(Number.POSITIVE_INFINITY, Number.POSITIVE_INFINITY, 0, 0);
};

APT.Rect.Infinite = function() {
    return APT.Rect.New(0, 0, Number.POSITIVE_INFINITY, Number.POSITIVE_INFINITY);
};

APT.Rect.Zero = function() {
    return new APT.Rect(APT.Point.Zero(), APT.Size.Zero());
};


