/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");

APT.SearchBar = Class.$define("APT.SearchBar", APT.View, {
    __init__: function() {
        this._delegate = null;
        this._showsCancelButton = false;
        this._cancelButton = null;
        this._textInput = null;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.SearchBar();
        copyObject._delegate = this._delegate;
        copyObject._showsCancelButton = this._showsCancelButton;
        copyObject._cancelButton = this._cancelButton;
        copyObject._textInput = this._textInput;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<fieldset class='style_markup_searchBar'><input type=\"text\"></fieldset>"));
    },

    setJQElement: function(jqelement) {
        this._jqElement = jqelement;

        this._textInput = this.jQElement().find("input").textinput();
        this._textInput.addClass("apt_searchBar_text");

        if (this._jqElement.children("a").length > 0)
        {
            this._cancelButton = this._jqElement.children("a").eq(0).button();
        }
        else
        {
            this._cancelButton = $("<a data-role=\"button\" ></a>").button();
            this._cancelButton.buttonMarkup({"mini" : true, "icon" : "delete", "iconpos" : "notext"});
        }

        this._cancelButton.button();
        this._cancelButton.buttonMarkup({"inline" : true});

        var thisSearchBar = this;
        this._cancelButton.on("vmouseup", function(e) {
            thisSearchBar.cancelBtnClicked(e);
        });

        var targetObject = this;

        this.jQElement().find("input").on("change", function(event) {
            targetObject["searchValueChange"](event);
            event.stopPropagation();
        });

        this.jQElement().find("input").on("keydown", function(event) {
            if (event.keyCode === 13)
            {
                targetObject["searchValueChange"](event);
                event.stopPropagation();
            }
        });

        this.setProperties();
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
    },

    delegate: function() {
        return this._delegate;
    },

    setText: function(aText) {
        this.jQElement().find("input").val(aText);
    },

    text: function() {
        return this.jQElement().find("input").val();
    },

    setPlaceholder: function(aText) {
        this.jQElement().find("input").attr("placeholder", aText);
    },

    placeholder: function() {
        if (this._textInput.attr("placeholder"))
        {
            return this._textInput.attr("placeholder");
        }

        return null;
    },

    setShowsCancelButton: function(aValue) {
        if (aValue && this._jqElement.find("a").length === 0)
        {
            this._cancelButton.appendTo(this._jqElement);
        }
        else
        {
            this._cancelButton.detach();
        }

        this.setNeedsLayout();
    },

    showsCancelButton: function() {
        return this._jqElement.children("a").length > 0;
    },

    cancelBtnClicked: function(ev) {
        if (this.delegate() !== null)
        {
            this.delegate().searchBarCancelButtonClicked(this);
        }
    },

    searchValueChange: function(ev) {
        if (this.delegate() !== null)
        {
            this.delegate().searchBar_textDidChange(this, this.text());
        }
    },

    processNode: function(markup, element) {
        var input = element.find("input");

        markup.find("input").attr("placeholder", input.attr("placeholder"));
        markup.find("input").val(input.attr("value"));

        this.setJQElement(markup);
    },

    layoutSubviews: function() {
        if (this._jqElement.find("a").length > 0)
        {
            var thisWith = parseInt(this._jqElement.css("width"));
            var thisPaddingLeft = parseInt(this._jqElement.css("padding-left"));
            var thisPaddingRight = parseInt(this._jqElement.css("padding-right"));

            var buttonWith = parseInt(this._cancelButton.css("width"));
            var buttonPaddingLeft = parseInt(this._cancelButton.css("padding-left"));
            var buttonPaddingRight = parseInt(this._cancelButton.css("padding-right"));
            var buttonMarginLeft = parseInt(this._cancelButton.css("margin-left"));
            var buttonMarginRight = parseInt(this._cancelButton.css("margin-right"));

            var textInputMarginLeft = parseInt(this._textInput.css("margin-left"));
            var textInputMarginRight = parseInt(this._textInput.css("margin-right"));
            var textInputWith = thisWith - thisPaddingLeft - thisPaddingRight - textInputMarginLeft - textInputMarginRight;
            textInputWith = textInputWith - buttonWith - buttonPaddingLeft - buttonPaddingRight - buttonMarginLeft - buttonMarginRight;
            this._textInput.css("width", new Number(textInputWith).toString() + "px");
        }
        else
        {
            this._textInput.css("width", "");
        }
    },

});


