/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");
APT.Control = Class.$define("APT.Control", APT.View);

APT.SegmentButton = Class.$define("APT.SegmentButton", APT.Control, {
    __init__: function() {
        this._label = null;
        this._image = null;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.SegmentButton();
        copyObject._label = this._label;
        copyObject._image = this._image;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<a data-role='button' data-theme='c' ></a>"));
    },

    setJQElement: function(aObject) {
        var text = aObject.text();
        this._jqElement = aObject;

        this._label = new APT.Label();
        if (text.length > 0)
        {
            this._label.setText(text);
            this.jQElement().text("");
        }

        this.jQElement().append(this._label.jQElement());
        this.jQElement().button();
        this.standardizeMarginAndPaddingFormInnerSpan();
        this.jQElement().addClass("apt_button");
        this.jQElement().addClass("apt_font");
        this.setProperties();
        this.jQElement().css("background-color", "");
        this.jQElement().buttonMarkup({"corners" : false});
        this.jQElement().buttonMarkup({"shadow" : false});
    },

    setTitle: function(titleText) {
        if (this._label !== null)
        {
            this._label.setText(titleText);
            this.setImage(null);
        }
    },

    title: function() {
        return this._label.text();
    },

    setImage: function(aImage) {
        this._image = aImage;
        if (aImage !== null)
        {
            this._jqElement.css("background-position", "initial initial");
            this._jqElement.css("background-image", "url(" + aImage.name() + ")");
            this._jqElement.css("background-repeat", "no-repeat");
            this._jqElement.css("background-position", "center center");
            this._jqElement.css("background-size", "contain");
            this._label.setText("");
        }
        else
        {
            this._jqElement.css("background-image", "none");
        }
    },

    image: function() {
        return this._image;
    },

    viewDidAppear: function(animated) {
        this.setNeedsLayout();

        this._label.viewDidAppear(animated);
        this.$super(animated);
    },

    layoutSubviews: function() {
        this.$super();

        this.standardizeMarginAndPaddingFormInnerSpan();
        if (this._label !== null)
        {
            this.adjustLabelSize();
        }
    },

    setEnabled: function(enabled) {
        this.$super(enabled);
        if (enabled)
        {
            this.setAlpha(1);
        }
        else
        {
            this.setAlpha(0.5);
        }
    },

    adjustLabelSize: function() {
        if (this._label !== null)
        {
            var buttonInnerSpanPaddingTop = parseInt(this.jQElement().find(".ui-btn-inner").css("padding-top"));
            var buttonInnerSpanPaddingBottom = parseInt(this.jQElement().find(".ui-btn-inner").css("padding-bottom"));
            var tempHeight = parseInt(this.jQElement().css("height"));
            var labelHeight = tempHeight - (buttonInnerSpanPaddingTop + buttonInnerSpanPaddingBottom);
            var labelHeightStr = new Number(labelHeight).toString() + "px";
            this._label.jQElement().css("height", labelHeightStr);

            this._label.setNeedsLayout();
        }
    },

    standardizeMarginAndPaddingFormInnerSpan: function() {
        var buttonInner = this.jQElement().children(".ui-btn-inner");
        buttonInner.css("margin-top", "0px");
        buttonInner.css("margin-bottom", "0px");
        buttonInner.css("padding-top", "9px");
        buttonInner.css("padding-bottom", "9px");
        buttonInner.css("border-top", "0px solid white");
    },

});


