/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");
APT.Control = Class.$define("APT.Control", APT.View);

APT.Slider = Class.$define("APT.Slider", APT.Control, {
    __init__: function() {
        this._continuous = true;
        this._targetObject = new Array();
        this._selector = new Array();
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.Slider();
        copyObject._continuous = this._continuous;
        copyObject._targetObject = this._targetObject;
        copyObject._selector = this._selector;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<div><input type=\"range\" value=\"0\" min=\"0\" max=\"1\" data-highlight=\"true\" style=\"display:none;\"/></div>"));
    },

    setJQElement: function(jqelement) {
        this._jqElement = jqelement.trigger("create");
        this.jQElement().find("input").slider();
        this.setProperties();
    },

    setValue: function(value) {
        for (var i = 0; i < this._selector.length; i++)
        {
            this.removeTarget_action_forControlEventsPrivate(this._targetObject[i], this._selector[i], APT.ControlEvent.ValueChanged);
        }

        var jqSlider = this.jQElement().find("input");
        jqSlider.val(new String(value));
        jqSlider.slider("refresh");
        for (var i = 0; i < this._selector.length; i++)
        {
            this.addTarget_action_forControlEventsPrivate(this._targetObject[i], this._selector[i], APT.ControlEvent.ValueChanged);
        }
    },

    value: function() {
        var jqSlider = this.jQElement().find("input");
        return parseFloat(jqSlider.val());
    },

    minimumValue: function() {
        var jqSlider = this.jQElement().find("input");
        return parseFloat(jqSlider.attr("min"));
    },

    setMinimumValue: function(minimumValue) {
        var jqSlider = this.jQElement().find("input");
        jqSlider.attr("min", new String(minimumValue));
        jqSlider.slider("refresh");
    },

    maximumValue: function() {
        var jqSlider = this.jQElement().find("input");
        return parseFloat(jqSlider.attr("max"));
    },

    setMaximumValue: function(maximumValue) {
        var jqSlider = this.jQElement().find("input");
        jqSlider.attr("max", new String(maximumValue));
        jqSlider.slider("refresh");
    },

    continuous: function() {
        return this._continuous;
    },

    setContinuous: function(continuous) {
        this._continuous = continuous;
        for (var i = 0; i < this._selector.length; i++)
        {
            this.removeTarget_action_forControlEventsPrivate(this._targetObject[i], this._selector[i], APT.ControlEvent.ValueChanged);
        }

        for (var i = 0; i < this._selector.length; i++)
        {
            this.addTarget_action_forControlEventsPrivate(this._targetObject[i], this._selector[i], APT.ControlEvent.ValueChanged);
        }
    },

    addTarget_action_forControlEvents: function(targetObject, selector, controlEvents) {
        if (APT.ControlEvent.ValueChanged & controlEvents)
        {
            this._targetObject.push(targetObject);
            this._selector.push(selector);
        }

        if (!this._continuous)
        {
            var eventHandler = function(event) {
                if (jQuery(this).attr("disable") !== "disable")
                {
                    targetObject[selector.valueOf()](event);
                }
            };

            if (selector !== null && selector !== undefined && selector.valueOf() !== "")
            {
                var eventName = "." + selector;
                if (APT.ControlEvent.ValueChanged & controlEvents)
                {
                    var children = this.jQElement().find(".ui-slider");
                    this.jQElement().on("vmouseup" + eventName, eventHandler);
                    children.on("vmouseup" + eventName, eventHandler);
                }
                else
                {
                    this.$super(targetObject, selector, controlEvents);
                }
            }
        }
        else
        {
            this.$super(targetObject, selector, controlEvents);
        }
    },

    sendActionsForControlEvents: function(controlEvents) {
        if (!this._continuous)
        {
            if (APT.ControlEvent.ValueChanged & controlEvents)
            {
                this.jQElement().trigger("vmouseup");
            }
        }
        else
        {
            this.$super(controlEvents);
        }
    },

    removeTarget_action_forControlEvents: function(targetObject, selector, controlEvents) {
        if (this._selector.length === 0 && selector !== null && selector !== undefined && selector.valueOf() !== "")
        {
            this.$super(targetObject, selector, controlEvents);
        }

        for (var i = 0; i < this._selector.length; i++)
        {
            if (this._selector[i] === selector && this._targetObject[i] === targetObject)
            {
                if (!this._continuous && (APT.ControlEvent.ValueChanged & controlEvents))
                {
                    if (selector !== null && selector !== undefined && selector.valueOf() !== "")
                    {
                        var eventName = "." + selector;
                        var children = this.jQElement().find(".ui-slider");
                        this.jQElement().off("vmouseup" + eventName);
                        children.off("vmouseup" + eventName);
                    }
                }
                else
                {
                    this.$super(targetObject, selector, controlEvents);
                }

                this._selector.splice(i, 1);
                this._targetObject.splice(i, 1);
            }
        }
    },

    addTarget_action_forControlEventsPrivate: function(targetObject, selector, controlEvents) {
        if (!this._continuous)
        {
            var eventHandler = function(event) {
                if (jQuery(this).attr("disable") !== "disable")
                {
                    targetObject[selector.valueOf()](event);
                }
            };

            if (selector !== null && selector !== undefined && selector.valueOf() !== "")
            {
                var eventName = "." + selector;
                if (APT.ControlEvent.ValueChanged & controlEvents)
                {
                    var children = this.jQElement().find(".ui-slider");
                    this.jQElement().on("vmouseup" + eventName, eventHandler);
                    children.on("vmouseup" + eventName, eventHandler);
                }
                else
                {
                    APT.Control.prototype.addTarget_action_forControlEvents.call(this, targetObject, selector, controlEvents);
                }
            }
        }
        else
        {
            APT.Control.prototype.addTarget_action_forControlEvents.call(this, targetObject, selector, controlEvents);
        }
    },

    removeTarget_action_forControlEventsPrivate: function(targetObject, selector, controlEvents) {
        if (!this._continuous)
        {
            if (selector !== null && selector !== undefined && selector.valueOf() !== "")
            {
                var eventName = "." + selector;
                if (APT.ControlEvent.ValueChanged & controlEvents)
                {
                    var children = this.jQElement().find(".ui-slider");
                    this.jQElement().off("vmouseup" + eventName);
                    children.off("vmouseup" + eventName);
                }
                else
                {
                    APT.Control.prototype.removeTarget_action_forControlEvents.call(this, targetObject, selector, controlEvents);
                }
            }
        }
        else
        {
            APT.Control.prototype.removeTarget_action_forControlEvents.call(this, targetObject, selector, controlEvents);
        }
    },

    processNode: function(markup, element) {
        var input = element.find("input");

        markup.find("input").attr("max", input.attr("max"));
        markup.find("input").attr("min", input.attr("min"));
        markup.find("input").val(input.attr("value"));

        this.setJQElement(markup);
    },

    setUserInteractionEnabled: function(enabled) {
        if (enabled)
        {
            this.jQElement().find("input").slider("enable");
        }
        else
        {
            this.jQElement().find("input").slider("disable");
        }
    },

    isUserInteractionEnabled: function() {
        return !this.jQElement().find("input").slider("option", "disabled");
    },

});


