/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.ViewController = Class.$define("APT.ViewController");

APT.TabBarController = Class.$define("APT.TabBarController", APT.ViewController, {
    __init__: function() {
        this._delegate = null;
        this._tabBar = new APT.TabBar();
        this._viewControllers = null;
        this._selectedViewController = null;
        this._maxTabBarItems = 5;
        this._moreNavigationController = null;
        this._moreListController = null;
        this.$super();
        this._moreListController = new APT.MoreListController();
        this._moreNavigationController = new APT.MoreNavigationController();
        this._moreNavigationController.initWithRootViewController(this._moreListController);
    },

    $copy: function() {
        var copyObject = new APT.TabBarController();
        copyObject._delegate = this._delegate;
        copyObject._tabBar = this._tabBar;
        copyObject._viewControllers = this._viewControllers;
        copyObject._selectedViewController = this._selectedViewController;
        copyObject._maxTabBarItems = this._maxTabBarItems;
        copyObject._moreNavigationController = this._moreNavigationController;
        copyObject._moreListController = this._moreListController;
        return copyObject;
    },

    moreNavigationController: function() {
        return this._moreNavigationController;
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
    },

    delegate: function() {
        return this._delegate;
    },

    tabBar: function() {
        return this._tabBar;
    },

    viewControllers: function() {
        if (this._viewControllers !== null && this._viewControllers !== undefined)
        {
            return this._viewControllers.slice(0);
        }
        else
        {
            return null;
        }
    },

    setViewControllers: function(viewControllers) {
        if (viewControllers !== null && viewControllers !== undefined)
        {
            this._viewControllers = viewControllers.slice(0);
            this._moreListController.setViewControllers(this._viewControllers);
            this._moreNavigationController.popToRootViewControllerAnimated(false);
            var items = new Array();
            if (viewControllers.length > this._maxTabBarItems)
            {
                for (var i = 0; i < this._maxTabBarItems - 1; i++)
                {
                    (this._viewControllers[i]).setTabBarController(this);
                    items.push((this._viewControllers[i]).tabBarItem());
                }

                items.push(this._moreNavigationController.tabBarItem());
            }
            else
            {
                for (var i = 0; i < this._viewControllers.length; i++)
                {
                    (this._viewControllers[i]).setTabBarController(this);
                    items.push((this._viewControllers[i]).tabBarItem());
                }
            }

            this._tabBar.setItems(items);
            this.setSelectedIndex(0);
        }
    },

    selectedViewController: function() {
        return this._selectedViewController;
    },

    setSelectedViewController: function(selectedViewController) {
        var index = 0;

        if (this.isSelectableViewController(selectedViewController))
        {
            this.selectController(selectedViewController);
        }
        else
        {
            if (this.hasViewControllers())
            {
                this._selectedViewController = this._viewControllers[0];
                this._tabBar.setSelectedItem(this._selectedViewController.tabBarItem());
                this.showContentViewController(this._selectedViewController, true);
                if (selectedViewController === null)
                {
                    this._selectedViewController = null;
                }
            }
            else
            {
                this._selectedViewController = null;
            }
        }
    },

    selectedIndex: function() {
        var result = APT.Global.NotFound;
        if (this.hasViewControllers() && this._selectedViewController !== null && this._selectedViewController !== undefined)
        {
            result = this._viewControllers.indexOf(this._selectedViewController);
        }

        return result;
    },

    setSelectedIndex: function(selectedIndex) {
        if (this.hasViewControllers())
        {
            if (selectedIndex < 0 || selectedIndex >= this._viewControllers.length)
            {
                selectedIndex = 0;
            }

            if (this._viewControllers.length > this._maxTabBarItems && selectedIndex >= this._maxTabBarItems - 1)
            {
                this._tabBar.setSelectedItem(this._moreNavigationController.tabBarItem());
                this._selectedViewController = this._viewControllers[selectedIndex];
                this._moreNavigationController.popToRootViewControllerAnimated(false);
                this._moreNavigationController.pushViewController_animated(this._selectedViewController, false);
                this.showContentViewController(this._moreNavigationController, true);
            }
            else
            {
                this._tabBar.setSelectedItem((this._viewControllers[selectedIndex]).tabBarItem());
                this._selectedViewController = this._viewControllers[selectedIndex];
                this.showContentViewController(this._selectedViewController, true);
            }
        }
    },

    tabBar_didSelectItem: function(tabBar, item) {
        this.setSelectedIndexNotifyingDelegate(tabBar, item);
    },

    loadView: function() {
        this.setView(new APT.View());
        this.view().jQElement().addClass("apt_tabBarControllerView");
        var screenBounds = APT.Screen.MainScreen().bounds();
        this.view().initWithFrame(screenBounds);
        var tabBarY = screenBounds.size.height - APT.TabBarController._tabBarDefaultHeight;
        this._tabBar.initWithFrame(APT.Rect.New(0, tabBarY, screenBounds.size.width, APT.TabBarController._tabBarDefaultHeight));
        this._tabBar.setDelegate(this);
        this.view().addSubview(this._tabBar);
    },

    viewDidAppear: function(animated) {
        var selectedViewController = this.selectedViewController();
        if (selectedViewController !== null && selectedViewController !== undefined)
        {
            if (this._viewControllers.length > this._maxTabBarItems && this._viewControllers.indexOf(selectedViewController) >= this._maxTabBarItems - 1)
            {
                this.showContentViewController(this._moreNavigationController, false);
            }
            else
            {
                this.showContentViewController(selectedViewController, false);
            }
        }
        else
        {
            if (this.hasViewControllers())
            {
                this._tabBar.setSelectedItem((this._viewControllers[0]).tabBarItem());
                this.showContentViewController(this._viewControllers[0], false);
            }
        }
    },

    isSelectableViewController: function(viewController) {
        var result = false;
        if (this.hasViewControllers() && viewController !== null && viewController !== undefined)
        {
            result = this._viewControllers.indexOf(viewController) >= 0 || this._moreNavigationController === viewController;
        }

        return result;
    },

    hasViewControllers: function() {
        return this._viewControllers !== null && this._viewControllers !== undefined && this._viewControllers.length > 0;
    },

    showContentViewController: function(viewController, checkViewInDom) {
        if (!checkViewInDom || this.view().isInDom())
        {
            var instanceOfTabBar = false;
            for (var i = 0; i < this.view().subviews().length; i++)
            {
                instanceOfTabBar = this.view().subviews()[i] instanceof APT.TabBar;
                if (!instanceOfTabBar)
                {
                    (this.view().subviews()[i]).removeFromSuperview();
                }
            }

            if (viewController.view() !== null && viewController.view() !== undefined)
            {
                var contentHeight = this.view().frame().size.height - APT.TabBarController._tabBarDefaultHeight;
                viewController.view().setFrame(APT.Rect.New(0, 0, this.view().frame().size.width, contentHeight));
                this.view().addSubview(viewController.view());
            }
        }
    },

    existDelegate: function() {
        return this._delegate !== null && this._delegate !== undefined;
    },

    existDelegateShouldSelectViewControllerMethod: function() {
        return this._delegate["tabBarController_shouldSelectViewController"] instanceof Function;
    },

    mustActivateViewController: function(viewController) {
        return !this.existDelegateShouldSelectViewControllerMethod() || this._delegate.tabBarController_shouldSelectViewController(this, viewController);
    },

    delegateHasDidSelectViewControllerMethod: function() {
        return this._delegate["tabBarController_didSelectViewController"] instanceof Function;
    },

    setSelectedIndexNotifyingDelegate: function(tabBar, item) {
        var index = tabBar.items().indexOf(item);
        var viewController = null;

        if (this._moreNavigationController.tabBarItem() === item)
        {
            viewController = this._moreNavigationController;
        }
        else
        {
            viewController = this.viewControllers()[index];
        }

        if (this.existDelegate())
        {
            if (this.mustActivateViewController(viewController))
            {
                this.setSelectedViewController(viewController);
                if (this.delegateHasDidSelectViewControllerMethod())
                {
                    this._delegate.tabBarController_didSelectViewController(this, viewController);
                }
            }
            else
            {
                viewController = this.selectedViewController();
                if (viewController === null || viewController === undefined)
                {
                    viewController = this.viewControllers()[0];
                }

                tabBar.setSelectedItem(viewController.tabBarItem());
            }
        }
        else
        {
            this.setSelectedViewController(viewController);
        }
    },

    selectController: function(viewController) {
        if (viewController === this._moreNavigationController)
        {
            this._tabBar.setSelectedItem(this._moreNavigationController.tabBarItem());
            if (this._moreNavigationController.topViewController() === this._moreListController)
            {
                this._selectedViewController = this._moreNavigationController;
            }
            else
            {
                this._selectedViewController = this._moreNavigationController.topViewController();
            }

            this.showContentViewController(this._moreNavigationController, true);
        }
        else
        {
            if (this._viewControllers.length > this._maxTabBarItems && this._viewControllers.indexOf(viewController) >= this._maxTabBarItems - 1)
            {
                this._tabBar.setSelectedItem(this._moreNavigationController.tabBarItem());
                this._selectedViewController = viewController;
                this._moreNavigationController.popToRootViewControllerAnimated(false);
                this._moreNavigationController.pushViewController_animated(viewController, false);
                this.showContentViewController(this._moreNavigationController, true);
            }
            else
            {
                this._tabBar.setSelectedItem(viewController.tabBarItem());
                this._selectedViewController = viewController;
                this.showContentViewController(this._selectedViewController, true);
            }
        }
    },

});

APT.TabBarController._tabBarDefaultHeight = 49;


