/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");
APT.Control = Class.$define("APT.Control", APT.View);

APT.TextField = Class.$define("APT.TextField", APT.Control, {
    __init__: function() {
        this._textColor = APT.Color.Black;
        this._font = new APT.Font();
        this._delegate = null;
        this._inputDelegate = null;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.TextField();
        copyObject._textColor = this._textColor;
        copyObject._font = this._font;
        copyObject._delegate = this._delegate;
        copyObject._inputDelegate = this._inputDelegate;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<input type='text' value=''/>"));
    },

    setJQElement: function(jqelement) {
        this._jqElement = jqelement.textinput();
        this.jQElement().addClass("apt_font");
        this.jQElement().addClass("apt_textField");

        if (this.borderStyle() < 0)
        {
            this.setBorderStyle(APT.TextFieldBorderStyle.None);
        }

        this.setProperties();

        this.jQElement().css("padding", "0px 5px");
        this.jQElement().css("margin", "0 0");
        APT.MenuController.sharedMenuController().clickEvent(this);
        this.setCountMenu();
    },

    value: function() {
        return this.jQElement().val();
    },

    setValue: function(aText) {
        this.jQElement().val(aText);
    },

    textAlignment: function() {
        return APT.TextAlignment.parseCSSTextAlign(this.jQElement().css("text-align"));
    },

    setTextAlignment: function(anAlignment) {
        this.jQElement().css("text-align", APT.TextAlignment.toCSSTextAlign(anAlignment));
    },

    keyboardType: function() {
        var keyboardType = this.jQElement().attr("type");
        switch(keyboardType.valueOf())
        {
            case "text":
                return APT.KeyboardType.Default;
                break;
            case "tel":
                return APT.KeyboardType.PhonePad;
                break;
            case "number":
                return APT.KeyboardType.NumberPad;
                break;
            case "url":
                return APT.KeyboardType.URL;
                break;
            case "email":
                return APT.KeyboardType.EmailAddress;
                break;
        }
    },

    setKeyboardType: function(keyboardType) {
        var clone = this.jQElement().clone();

        switch(keyboardType)
        {
            case APT.KeyboardType.Default:
                clone.attr("type", "text");
                break;
            case APT.KeyboardType.PhonePad:
                clone.attr("type", "tel");
                break;
            case APT.KeyboardType.NumberPad:
                clone.attr("type", "number");
                break;
            case APT.KeyboardType.URL:
                clone.attr("type", "url");
                break;
            case APT.KeyboardType.EmailAddress:
                clone.attr("type", "email");
                break;
        }

        clone.insertBefore(this.jQElement());

        this.jQElement().remove();

        this._jqElement = clone.textinput();
    },

    borderStyle: function() {
        if (this.jQElement().hasClass("textFieldBorderStyleNone"))
        {
            return APT.TextFieldBorderStyle.None;
        }
        else
        {
            if (this.jQElement().hasClass("textFieldBorderStyleLine"))
            {
                return APT.TextFieldBorderStyle.Line;
            }
            else
            {
                if (this.jQElement().hasClass("textFieldBorderStyleBezel"))
                {
                    return APT.TextFieldBorderStyle.Bezel;
                }
                else
                {
                    if (this.jQElement().hasClass("textFieldBorderStyleRoundedRect"))
                    {
                        return APT.TextFieldBorderStyle.RoundedRect;
                    }
                    else
                    {
                        return -1;
                    }
                }
            }
        }
    },

    setBorderStyle: function(anBorderStyle) {
        this.jQElement().removeClass("textFieldBorderStyleNone");
        this.jQElement().removeClass("textFieldBorderStyleLine");
        this.jQElement().removeClass("textFieldBorderStyleBezel");
        this.jQElement().removeClass("textFieldBorderStyleRoundedRect");
        switch(anBorderStyle)
        {
            case APT.TextFieldBorderStyle.None:
                this.jQElement().addClass("textFieldBorderStyleNone");
                break;
            case APT.TextFieldBorderStyle.Line:
                this.jQElement().addClass("textFieldBorderStyleLine");
                break;
            case APT.TextFieldBorderStyle.Bezel:
                this.jQElement().addClass("textFieldBorderStyleBezel");
                break;
            case APT.TextFieldBorderStyle.RoundedRect:
                this.jQElement().addClass("textFieldBorderStyleRoundedRect");
                break;
        }
    },

    textColor: function() {
        return this._textColor.updateFromCSS(this.jQElement().css("color"));
    },

    setTextColor: function(aColor) {
        this._textColor = aColor;
        this.jQElement().css("color", aColor.getHtmlValue().valueOf());
    },

    placeHolder: function() {
        if (this.jQElement().attr("placeholder"))
        {
            return this.jQElement().attr("placeholder");
        }

        return null;
    },

    setPlaceHolder: function(aPlaceHolder) {
        this.jQElement().attr("placeholder", aPlaceHolder);
    },

    font: function() {
        return this._font.updateFromCSS(this.jQElement());
    },

    setFont: function(aFont) {
        this._font = aFont;
        this._font.updateCSS(this.jQElement());
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
        if (aDelegate !== null)
        {
            var textField = this;

            this.jQElement()[0].onkeydown = function(e) {
                var keyCode = e.keyCode;
                if (keyCode === 13)
                {
                    textField.delegate().textFieldShouldReturn(textField);
                }
            };
        }
    },

    delegate: function() {
        return this._delegate;
    },

    setInputDelegate: function(inputDelegate) {
        this._inputDelegate = inputDelegate;
    },

    inputDelegate: function() {
        return this._inputDelegate;
    },

    setEditable: function(editable) {
        if (editable)
        {
            this.jQElement().removeAttr("readonly");
        }
        else
        {
            this.jQElement().attr("readonly", "readonly");
        }
    },

    isEditable: function() {
        if (this.jQElement().attr("readonly"))
        {
            return false;
        }

        return true;
    },

    becomeFirstResponder: function() {
        this.jQElement().focus();
        return true;
    },

    resignFirstResponder: function() {
        if (this.jQElement().is(":focus"))
        {
            this.jQElement().blur();
            return true;
        }

        return false;
    },

    frame: function() {
        var point = APT.Point.updateFromCSS(this._jqElement);
        var size = APT.Size.updateFromCSS(this._jqElement);
        if (this.isInDom())
        {
            size.width = size.width + parseInt(this.jQElement().css("padding-left")) + parseInt(this.jQElement().css("padding-right")) + parseInt(this.jQElement().css("border-left-width")) + parseInt(this.jQElement().css("border-right-width"));
            size.height = size.height + parseInt(this.jQElement().css("padding-top")) + parseInt(this.jQElement().css("padding-bottom")) + parseInt(this.jQElement().css("border-top-width")) + parseInt(this.jQElement().css("border-bottom-width"));
        }

        return new APT.Rect(point, size);
    },

    setCountMenu: function() {

        this.jQElement().on("blur" + " _touchOff", function(event) {
            APT.MenuController.sharedMenuController()._cont = 0;
        });
    },

});


