/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");
APT.ScrollView = Class.$define("APT.ScrollView", APT.View);

APT.TextView = Class.$define("APT.TextView", APT.ScrollView, {
    __init__: function() {
        this._textColor = APT.Color.Black;
        this._font = new APT.Font();
        this._delegate = null;
        this._inputDelegate = null;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.TextView();
        copyObject._textColor = this._textColor;
        copyObject._font = this._font;
        copyObject._delegate = this._delegate;
        copyObject._inputDelegate = this._inputDelegate;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<textarea></textarea>"));
    },

    setJQElement: function(jqelement) {
        this._jqElement = jqelement.textinput();
        this.jQElement().addClass("apt_font");
        this.jQElement().addClass("apt_textView");
        APT.MenuController.sharedMenuController().clickEvent(this);
        this.setCountMenu();
        this.setProperties();
    },

    setFrame: function(aFrame) {
        aFrame = aFrame.$copy();
        this.$super(aFrame);
        this.jQElement().css("max-height", aFrame.size.height);
    },

    text: function() {
        return this.jQElement().html();
    },

    setText: function(aText) {
        if (aText !== undefined && aText !== null)
        {
            this.jQElement().html(aText.valueOf());
        }
    },

    isEditable: function() {
        if (this.jQElement().attr("readonly"))
        {
            return false;
        }

        return true;
    },

    setEditable: function(editable) {
        if (editable)
        {
            this.jQElement().removeAttr("readonly");
        }
        else
        {
            this.jQElement().attr("readonly", "readonly");
        }
    },

    textAlignment: function() {
        return APT.TextAlignment.parseCSSTextAlign(this.jQElement().css("text-align"));
    },

    setTextAlignment: function(anAlignment) {
        this.jQElement().css("text-align", APT.TextAlignment.toCSSTextAlign(anAlignment));
    },

    textColor: function() {
        return this._textColor.updateFromCSS(this.jQElement().css("color"));
    },

    setTextColor: function(aColor) {
        this._textColor = aColor;
        this.jQElement().css("color", aColor.getHtmlValue().valueOf());
    },

    font: function() {
        return this._font.updateFromCSS(this.jQElement());
    },

    setFont: function(aFont) {
        this._font = aFont;
        this._font.updateCSS(this.jQElement());
    },

    hasText: function() {
        var text = this.jQElement().html();
        if (text !== null && text.length > 0)
        {
            return true;
        }

        return false;
    },

    becomeFirstResponder: function() {
        this.jQElement().focus();
        return true;
    },

    resignFirstResponder: function() {
        if (this.jQElement().is(":focus"))
        {
            this.jQElement().blur();
            return true;
        }

        return false;
    },

    setContentSize: function(size) {
        size = size.$copy();
    },

    contentSize: function() {
        return new APT.Size(0, 0);
    },

    frame: function() {
        var point = APT.Point.updateFromCSS(this._jqElement);
        var size = APT.Size.updateFromCSS(this._jqElement);
        if (this.isInDom())
        {
            size.width = size.width + parseInt(this.jQElement().css("padding-left")) + parseInt(this.jQElement().css("padding-right")) + parseInt(this.jQElement().css("border-left-width")) + parseInt(this.jQElement().css("border-right-width"));
            size.height = size.height + parseInt(this.jQElement().css("padding-top")) + parseInt(this.jQElement().css("padding-bottom")) + parseInt(this.jQElement().css("border-top-width")) + parseInt(this.jQElement().css("border-bottom-width"));
        }

        return new APT.Rect(point, size);
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
    },

    delegate: function() {
        return this._delegate;
    },

    setInputDelegate: function(inputDelegate) {
        this._inputDelegate = inputDelegate;
    },

    inputDelegate: function() {
        return this._inputDelegate;
    },

    setCountMenu: function() {

        this.jQElement().on("blur" + " _touchOff", function(event) {
            APT.MenuController.sharedMenuController()._cont = 0;
        });
    },

});


