/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.XMLParser = Class.$define("APT.XMLParser", {
    __init__: function() {
        this._delegate = null;
        this._stringXML = null;
        this._saxParser = null;
        this._shouldProcessNamespaces = false;
        this._openTagsStack = null;
    },

    $copy: function() {
        var copyObject = new APT.XMLParser();
        copyObject._delegate = this._delegate;
        copyObject._stringXML = this._stringXML;
        copyObject._saxParser = this._saxParser;
        copyObject._shouldProcessNamespaces = this._shouldProcessNamespaces;
        copyObject._openTagsStack = this._openTagsStack;
        return copyObject;
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
    },

    delegate: function() {
        return this._delegate;
    },

    setData: function(data) {
        this._stringXML = String.NewWithDataUsingEncoding(data, APT.StringEncoding.ASCII);
    },

    parse: function() {
        this._configSaxParser();
        this._openTagsStack = new Array();
        if (this._delegate !== null && this._delegate !== undefined && this._delegate["parserDidStartDocument"] instanceof Function)
        {
            this._delegate.parserDidStartDocument(this);
        }

        this._saxParser.write(this._stringXML).close();
    },

    setShouldProcessNamespaces: function(shouldProcessNamespaces) {
        this._shouldProcessNamespaces = shouldProcessNamespaces;
    },

    shouldProcessNamespaces: function() {
        return this._shouldProcessNamespaces;
    },

    _configSaxParser: function() {
        this._saxParser = sax.parser(true, {"xmlns" : this._shouldProcessNamespaces});
        var $that = this;

        this._saxParser["ontext"] = function(text) {
            if ($that.delegate() && $that.delegate()["parser_foundCharacters"] instanceof Function)
            {
                $that.delegate().parser_foundCharacters($that, text);
            }
        };

        this._saxParser["onopentag"] = function(node) {
            $that._openTagsStack.push(node);
            if ($that.delegate() && $that.delegate()["parser_didStartElement_namespaceURI_qualifiedName_attributes"] instanceof Function)
            {
                var values = new Array();
                var keys = Object.keys(node["attributes"]);
                if ($that._shouldProcessNamespaces)
                {
                    for (var i = 0; i < keys.length; i++)
                    {
                        values.push(node["attributes"][keys[i]]["value"]);
                    }

                    var attributesDictionary = APT.Dictionary.dictionaryWithObjectsForKeys(values, keys);

                    $that.delegate().parser_didStartElement_namespaceURI_qualifiedName_attributes($that, node["local"], node["uri"], node["name"], attributesDictionary);
                }
                else
                {
                    for (var i = 0; i < keys.length; i++)
                    {
                        values.push(node["attributes"][keys[i]]);
                    }

                    var attributesDictionary = APT.Dictionary.dictionaryWithObjectsForKeys(values, keys);

                    $that.delegate().parser_didStartElement_namespaceURI_qualifiedName_attributes($that, node["name"], null, null, attributesDictionary);
                }
            }
        };

        this._saxParser["onclosetag"] = function(tagName) {
            var node = $that._openTagsStack.pop();
            if ($that.delegate() && $that.delegate()["parser_didEndElement_namespaceURI_qualifiedName"] instanceof Function)
            {
                if ($that._shouldProcessNamespaces)
                {
                    $that.delegate().parser_didEndElement_namespaceURI_qualifiedName($that, node["local"], node["uri"], node["name"]);
                }
                else
                {
                    $that.delegate().parser_didEndElement_namespaceURI_qualifiedName($that, node["name"], null, null);
                }
            }
        };

        this._saxParser["onend"] = function() {
            if ($that.delegate() && $that.delegate()["parserDidEndDocument"] instanceof Function)
            {
                $that.delegate().parserDidEndDocument($that);
            }
        };
    },

});


